# Copyright 2017 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from unittest import mock

from vmware_nsxlib.tests.unit.v3 import nsxlib_testcase
from vmware_nsxlib.tests.unit.v3 import test_constants as consts
from vmware_nsxlib.v3 import exceptions as nsxlib_exc
from vmware_nsxlib.v3 import load_balancer


app_profile_types = load_balancer.ApplicationProfileTypes
app_profiles = [app_profile_types.HTTP,
                app_profile_types.FAST_TCP,
                app_profile_types.FAST_UDP]
per_profile_types = load_balancer.PersistenceProfileTypes
per_profiles = [per_profile_types.COOKIE, per_profile_types.SOURCE_IP]
monitor_types = load_balancer.MonitorTypes
monitors = [monitor_types.HTTP, monitor_types.HTTPS, monitor_types.ICMP,
            monitor_types.PASSIVE, monitor_types.TCP, monitor_types.UDP]


class TestApplicationProfile(nsxlib_testcase.NsxClientTestCase):

    def test_create_application_profiles(self):
        fake_profile = consts.FAKE_APPLICATION_PROFILE.copy()
        for profile_type in app_profiles:
            body = {
                'display_name': fake_profile['display_name'],
                'description': fake_profile['description'],
                'resource_type': profile_type,
                'tags': consts.FAKE_TAGS
            }
            with mock.patch.object(self.nsxlib.client, 'create') as create:
                self.nsxlib.load_balancer.application_profile.create(
                    display_name=body['display_name'],
                    description=body['description'],
                    resource_type=body['resource_type'],
                    tags=consts.FAKE_TAGS)
                create.assert_called_with('loadbalancer/application-profiles',
                                          body)

    def test_create_fast_tcp_profiles(self):
        fake_profile = consts.FAKE_APPLICATION_PROFILE.copy()
        body = {
            'display_name': fake_profile['display_name'],
            'description': fake_profile['description'],
            'resource_type': app_profile_types.FAST_TCP,
            'close_timeout': 8,
            'ha_flow_mirroring_enabled': True,
            'idle_timeout': 1800,
            'tags': consts.FAKE_TAGS
        }
        with mock.patch.object(self.nsxlib.client, 'create') as create:
            self.nsxlib.load_balancer.application_profile.create(
                display_name=body['display_name'],
                description=body['description'],
                resource_type=body['resource_type'],
                close_timeout=body['close_timeout'],
                ha_flow_mirroring_enabled=body['ha_flow_mirroring_enabled'],
                idle_timeout=body['idle_timeout'],
                tags=consts.FAKE_TAGS)
            create.assert_called_with('loadbalancer/application-profiles',
                                      body)

    def test_create_fast_udp_profiles(self):
        fake_profile = consts.FAKE_APPLICATION_PROFILE.copy()
        body = {
            'display_name': fake_profile['display_name'],
            'description': fake_profile['description'],
            'resource_type': app_profile_types.FAST_UDP,
            'flow_mirroring_enabled': True,
            'idle_timeout': 1800,
            'tags': consts.FAKE_TAGS
        }
        with mock.patch.object(self.nsxlib.client, 'create') as create:
            self.nsxlib.load_balancer.application_profile.create(
                display_name=body['display_name'],
                description=body['description'],
                resource_type=body['resource_type'],
                flow_mirroring_enabled=body['flow_mirroring_enabled'],
                idle_timeout=body['idle_timeout'],
                tags=consts.FAKE_TAGS)
            create.assert_called_with('loadbalancer/application-profiles',
                                      body)

    def test_create_http_profiles(self):
        fake_profile = consts.FAKE_APPLICATION_PROFILE.copy()
        body = {
            'display_name': fake_profile['display_name'],
            'description': fake_profile['description'],
            'resource_type': app_profile_types.HTTP,
            'http_redirect_to': fake_profile['http_redirect_to'],
            'http_redirect_to_https': fake_profile['http_redirect_to_https'],
            'ntlm': fake_profile['ntlm'],
            'request_body_size': fake_profile['request_body_size'],
            'request_header_size': fake_profile['request_header_size'],
            'response_header_size': fake_profile['response_header_size'],
            'response_timeout': fake_profile['response_timeout'],
            'x_forwarded_for': fake_profile['x_forwarded_for'],
            'idle_timeout': fake_profile['idle_timeout'],
            'tags': consts.FAKE_TAGS
        }
        with mock.patch.object(self.nsxlib.client, 'create') as create:
            self.nsxlib.load_balancer.application_profile.create(
                display_name=body['display_name'],
                description=body['description'],
                resource_type=body['resource_type'],
                http_redirect_to=body['http_redirect_to'],
                http_redirect_to_https=body['http_redirect_to_https'],
                ntlm=body['ntlm'],
                request_body_size=body['request_body_size'],
                request_header_size=body['request_header_size'],
                response_header_size=body['response_header_size'],
                response_timeout=body['response_timeout'],
                x_forwarded_for=body['x_forwarded_for'],
                idle_timeout=body['idle_timeout'],
                tags=consts.FAKE_TAGS)
            create.assert_called_with('loadbalancer/application-profiles',
                                      body)

    def test_list_application_profiles(self):
        with mock.patch.object(self.nsxlib.client, 'list') as list_call:
            self.nsxlib.load_balancer.application_profile.list()
            list_call.assert_called_with(
                resource='loadbalancer/application-profiles')

    def test_get_application_profile(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_profile = consts.FAKE_APPLICATION_PROFILE.copy()
            self.nsxlib.load_balancer.application_profile.get(
                fake_profile['id'])
            get.assert_called_with(
                'loadbalancer/application-profiles/%s' % fake_profile['id'])

    def test_delete_application_profile(self):
        with mock.patch.object(self.nsxlib.client, 'delete') as delete:
            fake_profile = consts.FAKE_APPLICATION_PROFILE.copy()
            self.nsxlib.load_balancer.application_profile.delete(
                fake_profile['id'])
            delete.assert_called_with(
                'loadbalancer/application-profiles/%s' % fake_profile['id'],
                headers=None)


class TestPersistenceProfile(nsxlib_testcase.NsxClientTestCase):

    def test_create_persistence_profiles(self):
        fake_profile = consts.FAKE_PERSISTENCE_PROFILE.copy()
        for profile_type in per_profiles:
            body = {
                'display_name': fake_profile['display_name'],
                'description': fake_profile['description'],
                'resource_type': profile_type,
                'tags': consts.FAKE_TAGS
            }
            with mock.patch.object(self.nsxlib.client, 'create') as create:
                self.nsxlib.load_balancer.persistence_profile.create(
                    body['display_name'], body['description'],
                    consts.FAKE_TAGS, body['resource_type'])
                create.assert_called_with('loadbalancer/persistence-profiles',
                                          body)

    def test_list_persistence_profiles(self):
        with mock.patch.object(self.nsxlib.client, 'list') as list_call:
            self.nsxlib.load_balancer.persistence_profile.list()
            list_call.assert_called_with(
                resource='loadbalancer/persistence-profiles')

    def test_get_persistence_profile(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_profile = consts.FAKE_APPLICATION_PROFILE.copy()
            self.nsxlib.load_balancer.persistence_profile.get(
                fake_profile['id'])
            get.assert_called_with(
                'loadbalancer/persistence-profiles/%s' % fake_profile['id'])

    def test_delete_persistence_profile(self):
        with mock.patch.object(self.nsxlib.client, 'delete') as delete:
            fake_profile = consts.FAKE_PERSISTENCE_PROFILE.copy()
            self.nsxlib.load_balancer.persistence_profile.delete(
                fake_profile['id'])
            delete.assert_called_with(
                'loadbalancer/persistence-profiles/%s' % fake_profile['id'],
                headers=None)


class TestRule(nsxlib_testcase.NsxClientTestCase):
    def test_create_rule(self):
        fake_rule = consts.FAKE_RULE.copy()
        body = {
            'display_name': fake_rule['display_name'],
            'description': fake_rule['description'],
            'resource_type': fake_rule['resource_type'],
            'phase': fake_rule['phase'],
            'match_strategy': fake_rule['match_strategy'],
            'tags': consts.FAKE_TAGS
        }
        with mock.patch.object(self.nsxlib.client, 'create') as create:
            self.nsxlib.load_balancer.rule.create(**body)
            create.assert_called_with('loadbalancer/rules', body)

    def test_list_rules(self):
        with mock.patch.object(self.nsxlib.client, 'list') as list_call:
            self.nsxlib.load_balancer.rule.list()
            list_call.assert_called_with(resource='loadbalancer/rules')

    def test_get_rule(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_rule = consts.FAKE_RULE.copy()
            self.nsxlib.load_balancer.rule.get(fake_rule['id'])
            get.assert_called_with('loadbalancer/rules/%s' % fake_rule['id'])

    def test_delete_rule(self):
        with mock.patch.object(self.nsxlib.client, 'delete') as delete:
            fake_rule = consts.FAKE_RULE.copy()
            self.nsxlib.load_balancer.rule.delete(fake_rule['id'])
            delete.assert_called_with(
                'loadbalancer/rules/%s' % fake_rule['id'], headers=None)


class TestClientSslProfile(nsxlib_testcase.NsxClientTestCase):

    def test_create_client_ssl_profiles(self):
        fake_profile = consts.FAKE_CLIENT_SSL_PROFILE.copy()
        body = {
            'display_name': fake_profile['display_name'],
            'description': fake_profile['description'],
            'tags': consts.FAKE_TAGS
        }
        with mock.patch.object(self.nsxlib.client, 'create') as create:
            self.nsxlib.load_balancer.client_ssl_profile.create(
                body['display_name'], body['description'],
                consts.FAKE_TAGS)
            create.assert_called_with('loadbalancer/client-ssl-profiles', body)

    def test_list_client_ssl_profiles(self):
        with mock.patch.object(self.nsxlib.client, 'list') as list_call:
            self.nsxlib.load_balancer.client_ssl_profile.list()
            list_call.assert_called_with(
                resource='loadbalancer/client-ssl-profiles')

    def test_get_client_ssl_profile(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_profile = consts.FAKE_CLIENT_SSL_PROFILE.copy()
            self.nsxlib.load_balancer.client_ssl_profile.get(
                fake_profile['id'])
            get.assert_called_with(
                'loadbalancer/client-ssl-profiles/%s' % fake_profile['id'])

    def test_delete_client_ssl_profile(self):
        with mock.patch.object(self.nsxlib.client, 'delete') as delete:
            fake_profile = consts.FAKE_CLIENT_SSL_PROFILE.copy()
            self.nsxlib.load_balancer.client_ssl_profile.delete(
                fake_profile['id'])
            delete.assert_called_with(
                'loadbalancer/client-ssl-profiles/%s' % fake_profile['id'],
                headers=None)


class TestServerSslProfile(nsxlib_testcase.NsxClientTestCase):

    def test_create_server_client_ssl_profiles(self):
        fake_profile = consts.FAKE_SERVER_SSL_PROFILE.copy()
        body = {
            'display_name': fake_profile['display_name'],
            'description': fake_profile['description'],
            'tags': consts.FAKE_TAGS
        }
        with mock.patch.object(self.nsxlib.client, 'create') as create:
            self.nsxlib.load_balancer.server_ssl_profile.create(
                body['display_name'], body['description'],
                consts.FAKE_TAGS)
            create.assert_called_with('loadbalancer/server-ssl-profiles', body)

    def test_list_server_ssl_profiles(self):
        with mock.patch.object(self.nsxlib.client, 'list') as list_call:
            self.nsxlib.load_balancer.server_ssl_profile.list()
            list_call.assert_called_with(
                resource='loadbalancer/server-ssl-profiles')

    def test_get_server_ssl_profile(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_profile = consts.FAKE_SERVER_SSL_PROFILE.copy()
            self.nsxlib.load_balancer.server_ssl_profile.get(
                fake_profile['id'])
            get.assert_called_with(
                'loadbalancer/server-ssl-profiles/%s' % fake_profile['id'])

    def test_delete_server_ssl_profile(self):
        with mock.patch.object(self.nsxlib.client, 'delete') as delete:
            fake_profile = consts.FAKE_SERVER_SSL_PROFILE.copy()
            self.nsxlib.load_balancer.server_ssl_profile.delete(
                fake_profile['id'])
            delete.assert_called_with(
                'loadbalancer/server-ssl-profiles/%s' % fake_profile['id'],
                headers=None)


class TestMonitor(nsxlib_testcase.NsxClientTestCase):

    def test_create_monitors(self):
        fake_monitor = consts.FAKE_MONITOR.copy()
        for monitor_type in monitors:
            body = {
                'display_name': fake_monitor['display_name'],
                'description': fake_monitor['description'],
                'resource_type': monitor_type,
                'tags': consts.FAKE_TAGS
            }
            with mock.patch.object(self.nsxlib.client, 'create') as create:
                self.nsxlib.load_balancer.monitor.create(
                    body['display_name'], body['description'],
                    consts.FAKE_TAGS, body['resource_type'])
                create.assert_called_with('loadbalancer/monitors',
                                          body)

    def test_list_monitors(self):
        with mock.patch.object(self.nsxlib.client, 'list') as list_call:
            self.nsxlib.load_balancer.monitor.list()
            list_call.assert_called_with(resource='loadbalancer/monitors')

    def test_get_monitor(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_monitor = consts.FAKE_MONITOR.copy()
            self.nsxlib.load_balancer.monitor.get(fake_monitor['id'])
            get.assert_called_with(
                'loadbalancer/monitors/%s' % fake_monitor['id'])

    def test_delete_monitor(self):
        with mock.patch.object(self.nsxlib.client, 'delete') as delete:
            fake_monitor = consts.FAKE_MONITOR.copy()
            self.nsxlib.load_balancer.monitor.delete(fake_monitor['id'])
            delete.assert_called_with(
                'loadbalancer/monitors/%s' % fake_monitor['id'], headers=None)


class TestPool(nsxlib_testcase.NsxClientTestCase):

    def test_create_pool(self):
        fake_pool = consts.FAKE_POOL.copy()
        body = {
            'display_name': fake_pool['display_name'],
            'description': fake_pool['description'],
            'algorithm': fake_pool['algorithm'],
            'tags': consts.FAKE_TAGS
        }
        with mock.patch.object(self.nsxlib.client, 'create') as create:
            self.nsxlib.load_balancer.pool.create(
                body['display_name'], body['description'],
                consts.FAKE_TAGS, algorithm=body['algorithm'])
            create.assert_called_with('loadbalancer/pools',
                                      body)

    def test_list_pools(self):
        with mock.patch.object(self.nsxlib.client, 'list') as list_call:
            self.nsxlib.load_balancer.pool.list()
            list_call.assert_called_with(resource='loadbalancer/pools')

    def test_get_pool(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_profile = consts.FAKE_POOL.copy()
            self.nsxlib.load_balancer.pool.get(fake_profile['id'])
            get.assert_called_with(
                'loadbalancer/pools/%s' % fake_profile['id'])

    def test_delete_pool(self):
        with mock.patch.object(self.nsxlib.client, 'delete') as delete:
            fake_profile = consts.FAKE_POOL.copy()
            self.nsxlib.load_balancer.pool.delete(fake_profile['id'])
            delete.assert_called_with(
                'loadbalancer/pools/%s' % fake_profile['id'], headers=None)

    def test_remove_monitor_from_pool(self):
        fake_pool = consts.FAKE_POOL.copy()
        fake_pool['active_monitor_ids'] = [consts.FAKE_MONITOR_UUID]
        body = {'display_name': fake_pool['display_name'],
                'description': fake_pool['description'],
                'id': fake_pool['id'],
                'algorithm': fake_pool['algorithm'],
                'active_monitor_ids': []}
        with mock.patch.object(self.nsxlib.client, 'get',
                               return_value=fake_pool):
            with mock.patch.object(self.nsxlib.client, 'update') as update:
                self.nsxlib.load_balancer.pool.remove_monitor_from_pool(
                    fake_pool['id'], consts.FAKE_MONITOR_UUID)
                resource = 'loadbalancer/pools/%s' % fake_pool['id']
                update.assert_called_with(resource, body)

    def test_remove_non_exist_monitor_from_pool(self):
        fake_pool = consts.FAKE_POOL.copy()
        fake_pool['active_monitor_ids'] = [consts.FAKE_MONITOR_UUID]
        with mock.patch.object(self.nsxlib.client, 'get',
                               return_value=fake_pool):
            self.assertRaises(
                nsxlib_exc.ResourceNotFound,
                self.nsxlib.load_balancer.pool.remove_monitor_from_pool,
                fake_pool['id'],
                'xxx-yyy')

    def test_add_monitor_to_pool(self):
        fake_pool = consts.FAKE_POOL.copy()
        body = {'display_name': fake_pool['display_name'],
                'description': fake_pool['description'],
                'id': fake_pool['id'],
                'algorithm': fake_pool['algorithm'],
                'active_monitor_ids': [consts.FAKE_MONITOR_UUID]}
        with mock.patch.object(self.nsxlib.client, 'get',
                               return_value=fake_pool):
            with mock.patch.object(self.nsxlib.client, 'update') as update:
                self.nsxlib.load_balancer.pool.add_monitor_to_pool(
                    fake_pool['id'], consts.FAKE_MONITOR_UUID)
                resource = 'loadbalancer/pools/%s' % fake_pool['id']
                update.assert_called_with(resource, body)


class TestVirtualServer(nsxlib_testcase.NsxClientTestCase):

    def test_create_virtual_server(self):
        fake_virtual_server = consts.FAKE_VIRTUAL_SERVER.copy()
        body = {
            'display_name': fake_virtual_server['display_name'],
            'description': fake_virtual_server['description'],
            'ip_protocol': fake_virtual_server['ip_protocol'],
            'port': fake_virtual_server['port'],
            'enabled': fake_virtual_server['enabled'],
            'tags': consts.FAKE_TAGS
        }
        with mock.patch.object(self.nsxlib.client, 'create') as create:
            self.nsxlib.load_balancer.virtual_server.create(
                body['display_name'], body['description'],
                consts.FAKE_TAGS, ip_protocol=body['ip_protocol'],
                port=body['port'], enabled=body['enabled'])
            create.assert_called_with('loadbalancer/virtual-servers',
                                      body)

    def test_list_virtual_servers(self):
        with mock.patch.object(self.nsxlib.client, 'list') as list_call:
            self.nsxlib.load_balancer.virtual_server.list()
            list_call.assert_called_with(
                resource='loadbalancer/virtual-servers')

    def test_get_virtual_server(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_virtual_server = consts.FAKE_VIRTUAL_SERVER.copy()
            self.nsxlib.load_balancer.virtual_server.get(
                fake_virtual_server['id'])
            get.assert_called_with(
                'loadbalancer/virtual-servers/%s' % fake_virtual_server['id'])

    def test_delete_virtual_server(self):
        with mock.patch.object(self.nsxlib.client, 'delete') as delete:
            fake_virtual_server = consts.FAKE_VIRTUAL_SERVER.copy()
            self.nsxlib.load_balancer.virtual_server.delete(
                fake_virtual_server['id'])
            delete.assert_called_with(
                'loadbalancer/virtual-servers/%s' % fake_virtual_server['id'],
                headers=None)

    def test_add_rule(self):
        fake_virtual_server = consts.FAKE_VIRTUAL_SERVER.copy()
        body = {
            'display_name': fake_virtual_server['display_name'],
            'description': fake_virtual_server['description'],
            'id': fake_virtual_server['id'],
            'enabled': fake_virtual_server['enabled'],
            'port': fake_virtual_server['port'],
            'ip_protocol': fake_virtual_server['ip_protocol'],
            'rule_ids': [consts.FAKE_RULE_UUID]
        }
        with mock.patch.object(self.nsxlib.client, 'get') as mock_get, \
            mock.patch.object(self.nsxlib.client, 'update') as mock_update:
            mock_get.return_value = fake_virtual_server
            self.nsxlib.load_balancer.virtual_server.add_rule(
                fake_virtual_server['id'], consts.FAKE_RULE_UUID)
            mock_update.assert_called_with(
                'loadbalancer/virtual-servers/%s' % fake_virtual_server['id'],
                body)

    def test_remove_rule(self):
        fake_virtual_server = consts.FAKE_VIRTUAL_SERVER.copy()
        fake_virtual_server['rule_ids'] = [consts.FAKE_RULE_UUID]
        body = {
            'display_name': fake_virtual_server['display_name'],
            'description': fake_virtual_server['description'],
            'id': fake_virtual_server['id'],
            'enabled': fake_virtual_server['enabled'],
            'port': fake_virtual_server['port'],
            'ip_protocol': fake_virtual_server['ip_protocol'],
            'rule_ids': []
        }
        with mock.patch.object(self.nsxlib.client, 'get') as mock_get, \
            mock.patch.object(self.nsxlib.client, 'update') as mock_update:
            mock_get.return_value = fake_virtual_server
            self.nsxlib.load_balancer.virtual_server.remove_rule(
                fake_virtual_server['id'], consts.FAKE_RULE_UUID)
            mock_update.assert_called_with(
                'loadbalancer/virtual-servers/%s' % fake_virtual_server['id'],
                body)

    def test_add_client_ssl_profile_binding(self):
        fake_virtual_server = consts.FAKE_VIRTUAL_SERVER.copy()
        body = {
            'display_name': fake_virtual_server['display_name'],
            'description': fake_virtual_server['description'],
            'id': fake_virtual_server['id'],
            'enabled': fake_virtual_server['enabled'],
            'port': fake_virtual_server['port'],
            'ip_protocol': fake_virtual_server['ip_protocol'],
            'client_ssl_profile_binding': {
                'ssl_profile_id': consts.FAKE_CLIENT_SSL_PROFILE_UUID,
                'default_certificate_id': consts.FAKE_DEFAULT_CERTIFICATE_ID,
                'client_auth': 'IGNORE',
                'certificate_chain_depth': 3
            }
        }
        with mock.patch.object(self.nsxlib.client, 'get') as mock_get, \
            mock.patch.object(self.nsxlib.client, 'update') as mock_update:
            mock_get.return_value = fake_virtual_server
            vs_client = self.nsxlib.load_balancer.virtual_server
            vs_client.add_client_ssl_profile_binding(
                fake_virtual_server['id'],
                consts.FAKE_CLIENT_SSL_PROFILE_UUID,
                consts.FAKE_DEFAULT_CERTIFICATE_ID,
                client_auth='IGNORE',
                certificate_chain_depth=3,
                xyz='xyz'
            )
            mock_update.assert_called_with(
                'loadbalancer/virtual-servers/%s' % fake_virtual_server['id'],
                body)

    def test_add_server_ssl_profile_binding(self):
        fake_virtual_server = consts.FAKE_VIRTUAL_SERVER.copy()
        body = {
            'display_name': fake_virtual_server['display_name'],
            'description': fake_virtual_server['description'],
            'id': fake_virtual_server['id'],
            'enabled': fake_virtual_server['enabled'],
            'port': fake_virtual_server['port'],
            'ip_protocol': fake_virtual_server['ip_protocol'],
            'server_ssl_profile_binding': {
                'ssl_profile_id': consts.FAKE_SERVER_SSL_PROFILE_UUID,
                'server_auth': 'IGNORE',
                'certificate_chain_depth': 3
            }
        }
        with mock.patch.object(self.nsxlib.client, 'get') as mock_get, \
            mock.patch.object(self.nsxlib.client, 'update') as mock_update:
            mock_get.return_value = fake_virtual_server
            vs_client = self.nsxlib.load_balancer.virtual_server
            vs_client.add_server_ssl_profile_binding(
                fake_virtual_server['id'],
                consts.FAKE_SERVER_SSL_PROFILE_UUID,
                server_auth='IGNORE',
                certificate_chain_depth=3,
                xyz='xyz')
            mock_update.assert_called_with(
                'loadbalancer/virtual-servers/%s' % fake_virtual_server['id'],
                body)


class TestService(nsxlib_testcase.NsxClientTestCase):

    def test_create_service(self):
        fake_service = consts.FAKE_SERVICE.copy()
        body = {
            'display_name': fake_service['display_name'],
            'description': fake_service['description'],
            'enabled': fake_service['enabled'],
            'attachment': fake_service['attachment'],
            'relax_scale_validation': fake_service['relax_scale_validation'],
            'tags': consts.FAKE_TAGS
        }
        with mock.patch.object(self.nsxlib.client, 'create') as create, \
                mock.patch.object(self.nsxlib, 'feature_supported') as support:
            support.return_value = True
            self.nsxlib.load_balancer.service.create(
                body['display_name'], body['description'],
                consts.FAKE_TAGS, enabled=body['enabled'],
                attachment=body['attachment'],
                relax_scale_validation=body['relax_scale_validation'])
            create.assert_called_with('loadbalancer/services',
                                      body)

    def test_list_services(self):
        with mock.patch.object(self.nsxlib.client, 'list') as list_call:
            self.nsxlib.load_balancer.service.list()
            list_call.assert_called_with(resource='loadbalancer/services')

    def test_get_service(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_service = consts.FAKE_SERVICE.copy()
            self.nsxlib.load_balancer.service.get(fake_service['id'])
            get.assert_called_with(
                'loadbalancer/services/%s' % fake_service['id'])

    def test_get_stats(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_service = consts.FAKE_SERVICE.copy()
            self.nsxlib.load_balancer.service.get_stats(fake_service['id'])
            get.assert_called_with(
                'loadbalancer/services/%s/statistics?source=realtime' %
                fake_service['id'],
                silent=False)

    def test_get_status(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_service = consts.FAKE_SERVICE.copy()
            self.nsxlib.load_balancer.service.get_status(fake_service['id'])
            get.assert_called_with(
                'loadbalancer/services/%s/status' % fake_service['id'])

    def test_get_virtual_servers_status(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_service = consts.FAKE_SERVICE.copy()
            self.nsxlib.load_balancer.service.get_virtual_servers_status(
                fake_service['id'])
            get.assert_called_with(
                'loadbalancer/services/%s/virtual-servers/status' %
                fake_service['id'])

    def test_delete_service(self):
        with mock.patch.object(self.nsxlib.client, 'delete') as delete:
            fake_service = consts.FAKE_SERVICE.copy()
            self.nsxlib.load_balancer.service.delete(fake_service['id'])
            delete.assert_called_with(
                'loadbalancer/services/%s' % fake_service['id'], headers=None)

    def test_get_usage(self):
        with mock.patch.object(self.nsxlib.client, 'get') as get:
            fake_service = consts.FAKE_SERVICE.copy()
            self.nsxlib.load_balancer.service.get_usage(fake_service['id'])
            get.assert_called_with(
                'loadbalancer/services/%s/usage' % fake_service['id'])



# Copyright (c) 2015 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from oslo_utils import uuidutils

from vmware_nsxlib.tests.unit.v3 import nsxlib_testcase
from vmware_nsxlib.tests.unit.v3 import test_constants
from vmware_nsxlib.v3 import exceptions as nsxlib_exc
from vmware_nsxlib.v3 import nsx_constants as const


class TestNsxLibFirewallSection(nsxlib_testcase.NsxLibTestCase):
    """Tests for vmware_nsxlib.v3.security.NsxLibFirewallSection"""

    def setUp(self, *args, **kwargs):
        super(TestNsxLibFirewallSection, self).setUp(with_mocks=False)

    def test_get_logicalport_reference(self):
        mock_port = '3ed55c9f-f879-4048-bdd3-eded92465252'
        result = self.nsxlib.firewall_section.get_logicalport_reference(
            mock_port)
        expected = {
            'target_id': '3ed55c9f-f879-4048-bdd3-eded92465252',
            'target_type': 'LogicalPort'
        }
        self.assertEqual(expected, result)

    def test_get_rule_address(self):
        result = self.nsxlib.firewall_section.get_rule_address(
            'target-id', 'display-name')
        expected = {
            'target_display_name': 'display-name',
            'target_id': 'target-id',
            'is_valid': True,
            'target_type': 'IPv4Address'
        }
        self.assertEqual(expected, result)

    def test_get_l4portset_nsservice(self):
        result = self.nsxlib.firewall_section.get_l4portset_nsservice()
        expected = {
            'service': {
                'resource_type': 'L4PortSetNSService',
                'source_ports': [],
                'destination_ports': [],
                'l4_protocol': 'TCP'
            }
        }
        self.assertEqual(expected, result)

    def test_get_rule_dict(self):
        result = self.nsxlib.firewall_section.get_rule_dict(
            'display_name', sources='sources', destinations='destinations',
            direction=const.IN_OUT, ip_protocol=const.IPV4_IPV6,
            services='services', action=const.FW_ACTION_ALLOW,
            logged=True, disabled=True, applied_tos='applied_tos',
            rule_tag='rule_tag')
        expected = {'display_name': 'display_name',
                    'sources': 'sources',
                    'destinations': 'destinations',
                    'direction': const.IN_OUT,
                    'ip_protocol': const.IPV4_IPV6,
                    'services': 'services',
                    'action': const.FW_ACTION_ALLOW,
                    'logged': True,
                    'disabled': True,
                    'applied_tos': 'applied_tos',
                    'rule_tag': 'rule_tag'}
        self.assertEqual(expected, result)

    def test_create_rules_with_protocol(self):
        with mock.patch("vmware_nsxlib.v3.security.NsxLibFirewallSection"
                        ".add_rules") as add_rules:
            rule_id = uuidutils.generate_uuid()
            rule = {'id': rule_id,
                    'ethertype': 'IPv4',
                    'protocol': 'ipip',
                    'direction': 'ingress',
                    'remote_ip_prefix': None}
            rules = [rule]
            section_id = 'section-id'
            group_id = 'nsgroup-id'
            target_id = 'dummy'
            self.nsxlib.firewall_section.create_section_rules(
                section_id, group_id, False,
                "ALLOW", rules, {rule_id: target_id})
            add_rules.assert_called_once_with([
                {'display_name': mock.ANY,
                 'ip_protocol': 'IPV4',
                 'direction': 'IN',
                 'services': [{'service': {
                     'resource_type': 'IPProtocolNSService',
                     'protocol_number': 4}}],
                 'disabled': False,
                 'sources': [{'target_id': target_id,
                              'target_type': 'NSGroup'}],
                 'destinations': [{'target_id': group_id,
                                   'target_type': 'NSGroup'}],
                 'logged': False, 'action': 'ALLOW'}], section_id)

    def test_create_rules_ingress_with_port(self):
        with mock.patch("vmware_nsxlib.v3.security.NsxLibFirewallSection"
                        ".add_rules") as add_rules:
            rule_id = uuidutils.generate_uuid()
            rule = {'id': rule_id,
                    'ethertype': 'IPv4',
                    'protocol': 'tcp',
                    'direction': 'ingress',
                    'port_range_min': 80,
                    'port_range_max': 80,
                    'remote_ip_prefix': None}
            rules = [rule]
            section_id = 'section-id'
            group_id = 'nsgroup-id'
            target_id = 'dummy'
            self.nsxlib.firewall_section.create_section_rules(
                section_id, group_id, False,
                "ALLOW", rules, {rule_id: target_id})
            add_rules.assert_called_once_with([
                {'display_name': mock.ANY,
                 'ip_protocol': 'IPV4',
                 'direction': 'IN',
                 'services': [{'service': {
                     'l4_protocol': 'TCP',
                     'destination_ports': ['80'],
                     'source_ports': [],
                     'resource_type': 'L4PortSetNSService'}}],
                 'disabled': False,
                 'sources': [{'target_id': target_id,
                              'target_type': 'NSGroup'}],
                 'destinations': [{'target_id': group_id,
                                   'target_type': 'NSGroup'}],
                 'logged': False, 'action': 'ALLOW'}], section_id)

    def test_create_rules_egress_with_port(self):
        with mock.patch("vmware_nsxlib.v3.security.NsxLibFirewallSection"
                        ".add_rules") as add_rules:
            rule_id = uuidutils.generate_uuid()
            rule = {'id': rule_id,
                    'ethertype': 'IPv4',
                    'protocol': 'tcp',
                    'direction': 'egress',
                    'port_range_min': 80,
                    'port_range_max': 80,
                    'remote_ip_prefix': None}
            rules = [rule]
            section_id = 'section-id'
            group_id = 'nsgroup-id'
            target_id = 'dummy'
            self.nsxlib.firewall_section.create_section_rules(
                section_id, group_id, False,
                "ALLOW", rules, {rule_id: target_id})
            add_rules.assert_called_once_with([
                {'display_name': mock.ANY,
                 'ip_protocol': 'IPV4',
                 'direction': 'OUT',
                 'services': [{'service': {
                     'l4_protocol': 'TCP',
                     'destination_ports': ['80'],
                     'source_ports': [],
                     'resource_type': 'L4PortSetNSService'}}],
                 'disabled': False,
                 'destinations': [{'target_id': target_id,
                                   'target_type': 'NSGroup'}],
                 'sources': [{'target_id': group_id,
                              'target_type': 'NSGroup'}],
                 'logged': False, 'action': 'ALLOW'}], section_id)

    def test_create_rule_with_illegal_protocol(self):
        rule_id = uuidutils.generate_uuid()
        rule = {'id': rule_id,
                'ethertype': 'IPv4',
                'protocol': 'bad',
                'direction': 'ingress',
                'remote_ip_prefix': None}
        rules = [rule]
        self.assertRaises(nsxlib_exc.InvalidInput,
                          self.nsxlib.firewall_section.create_section_rules,
                          'section-id', 'nsgroup-id', False,
                          "ALLOW", rules, {rule_id: 'dummy'})

    def test_create_rule_with_icmp(self):
        nsx_ver = ["2.3.0", "2.4.0"]
        for nsx_ver in nsx_ver:
            with mock.patch("vmware_nsxlib.v3.security.NsxLibFirewallSection"
                            ".add_rules") as add_rules:
                with mock.patch("vmware_nsxlib.v3.NsxLib.get_version",
                                return_value=nsx_ver):
                    rule_id = uuidutils.generate_uuid()
                    rule = {'id': rule_id,
                            'ethertype': 'IPv4',
                            'protocol': 'icmp',
                            'direction': 'egress',
                            'port_range_min': 33,
                            'port_range_max': 0,
                            'remote_ip_prefix': None}
                    rules = [rule]
                    section_id = 'section-id'
                    group_id = 'nsgroup-id'
                    target_id = 'dummy'
                    self.nsxlib.firewall_section.create_section_rules(
                        section_id, group_id, False,
                        "ALLOW", rules, {rule_id: target_id})
                    add_rules.assert_called_once_with([
                        {'display_name': mock.ANY,
                            'ip_protocol': 'IPV4',
                            'direction': 'OUT',
                            'services': [{'service': {
                                'protocol': 'ICMPv4',
                                'icmp_type': 33,
                                'icmp_code': 0,
                                'resource_type': 'ICMPTypeNSService'}}],
                            'disabled': False,
                            'destinations': [{'target_id': target_id,
                                              'target_type': 'NSGroup'}],
                            'sources': [{'target_id': group_id,
                                        'target_type': 'NSGroup'}],
                            'logged': False, 'action': 'ALLOW'}], section_id)

    def test_create_rule_with_illegal_icmp(self):
        rule_id = uuidutils.generate_uuid()
        rule = {'id': rule_id,
                'ethertype': 'IPv4',
                'protocol': 'icmp',
                'direction': 'egress',
                'port_range_min': 2,
                'port_range_max': 3,
                'remote_ip_prefix': None}
        rules = [rule]
        section_id = 'section-id'
        group_id = 'nsgroup-id'
        target_id = 'dummy'
        with mock.patch("vmware_nsxlib.v3.NsxLib.get_version",
                        return_value="2.3.0"):
            self.assertRaises(
                nsxlib_exc.InvalidInput,
                self.nsxlib.firewall_section.create_section_rules,
                section_id, group_id, False,
                "ALLOW", rules, {rule_id: target_id})
        with mock.patch("vmware_nsxlib.v3.NsxLib.get_version",
                        return_value="2.4.0"):
            self.assertRaises(
                nsxlib_exc.InvalidInput,
                self.nsxlib.firewall_section.create_section_rules,
                section_id, group_id, False,
                "ALLOW", rules, {rule_id: target_id})

    def test_create_rule_with_illegal_icmp_2_4(self):
        rule_id = uuidutils.generate_uuid()
        rule = {'id': rule_id,
                'ethertype': 'IPv4',
                'protocol': 'icmp',
                'direction': 'egress',
                'port_range_min': 9,
                'port_range_max': 16,
                'remote_ip_prefix': None}
        rules = [rule]
        section_id = 'section-id'
        group_id = 'nsgroup-id'
        target_id = 'dummy'
        with mock.patch("vmware_nsxlib.v3.NsxLib.get_version",
                        return_value="2.4.0"):
            self.assertRaises(
                nsxlib_exc.InvalidInput,
                self.nsxlib.firewall_section.create_section_rules,
                section_id, group_id, False,
                "ALLOW", rules, {rule_id: target_id})

    def test_create_rule_with_illegal_icmp_0_255(self):
        rule_id = uuidutils.generate_uuid()
        rule = {'id': rule_id,
                'ethertype': 'IPv4',
                'protocol': 'icmp',
                'direction': 'egress',
                'port_range_min': 0,
                'port_range_max': 255,
                'remote_ip_prefix': None}
        rules = [rule]
        section_id = 'section-id'
        group_id = 'nsgroup-id'
        target_id = 'dummy'
        with mock.patch("vmware_nsxlib.v3.NsxLib.get_version",
                        return_value="2.4.0"):
            self.assertRaises(
                nsxlib_exc.InvalidInput,
                self.nsxlib.firewall_section.create_section_rules,
                section_id, group_id, False,
                "ALLOW", rules, {rule_id: target_id})

    def test_create_with_rules(self):
        expected_body = {
            'display_name': 'display-name',
            'description': 'section-description',
            'stateful': True,
            'section_type': "LAYER3",
            'applied_tos': [],
            'rules': [{
                'display_name': 'rule-name',
                'direction': 'IN_OUT',
                'ip_protocol': "IPV4_IPV6",
                'action': "ALLOW",
                'logged': False,
                'disabled': False,
                'sources': [],
                'destinations': [],
                'services': []
            }],
            'tags': []
        }
        with mock.patch.object(self.nsxlib.client, 'create') as create:
            rule = self.nsxlib.firewall_section.get_rule_dict('rule-name')
            self.nsxlib.firewall_section.create_with_rules(
                'display-name', 'section-description', rules=[rule])
            resource = 'firewall/sections?operation=insert_bottom' \
                '&action=create_with_rules'
            create.assert_called_with(resource, expected_body, headers=None)

    def test_set_rule_logging(self):
        section_id = '111'
        rule_id = 1
        orig_rule = {'id': rule_id, 'logged': False}
        with mock.patch.object(self.nsxlib.firewall_section, 'get_rules',
                               return_value={'results': [orig_rule]}),\
            mock.patch.object(self.nsxlib.client, 'get', return_value={}),\
            mock.patch.object(self.nsxlib.client, 'create') as update:
            self.nsxlib.firewall_section.set_rule_logging(section_id, True)
            update.assert_called_once_with(
                'firewall/sections/111?action=update_with_rules',
                {'rules': [{'id': 1, 'logged': True}]}, headers=None)

    def test_get_excludelist(self):
        with mock.patch.object(self.nsxlib.client, 'list') as clist:
            self.nsxlib.firewall_section.get_excludelist()
            clist.assert_called_with('firewall/excludelist')

    def test_add_to_excludelist(self):
        target_id = '111'
        target_type = const.NSGROUP
        with mock.patch.object(self.nsxlib.client, 'create') as create:
            self.nsxlib.firewall_section.add_member_to_fw_exclude_list(
                target_id, target_type)
            create.assert_called_once_with(
                'firewall/excludelist?action=add_member',
                {'target_id': target_id, 'target_type': target_type},
                headers=None)

    def test_del_from_excludelist(self):
        target_id = '111'
        target_type = const.NSGROUP
        with mock.patch.object(self.nsxlib.client, 'create') as create:
            self.nsxlib.firewall_section.remove_member_from_fw_exclude_list(
                target_id, target_type)
            create.assert_called_once_with(
                'firewall/excludelist?action=remove_member&'
                'object_id=%s' % target_id,
                None,
                headers=None)

    def test_update(self):
        fws_tags = [{"scope": "name", "tag": "new_name"}]
        with mock.patch.object(self.nsxlib.client, 'update') as update:
            with mock.patch.object(self.nsxlib.client, 'get') as get:
                get.return_value = {}
                self.nsxlib.firewall_section.update('fw_section_id',
                                                    tags_update=fws_tags)
                resource = 'firewall/sections/%s' % 'fw_section_id'
                data = {'tags': fws_tags}
                update.assert_called_with(resource, data, headers=None)

    def test_create_rules_using_add_rules(self):
        revision = 5
        with mock.patch("vmware_nsxlib.v3.NsxLib.get_version",
                        return_value='2.5.0'),\
            mock.patch.object(self.nsxlib.client, 'get',
                              return_value={'_revision': revision}),\
            mock.patch.object(self.nsxlib.client, 'create') as create:
            rule_id = uuidutils.generate_uuid()
            rule = {'id': rule_id,
                    'ethertype': 'IPv4',
                    'protocol': 'ipip',
                    'direction': 'ingress',
                    'remote_ip_prefix': None}
            rules = [rule]
            section_id = 'section-id'
            group_id = 'nsgroup-id'
            target_id = 'dummy'
            self.nsxlib.firewall_section.create_section_rules(
                section_id, group_id, False,
                "ALLOW", rules, {rule_id: target_id})
            expected_rule = {'display_name': mock.ANY,
                             'ip_protocol': 'IPV4',
                             'direction': 'IN',
                             'services': [{'service': {
                                 'resource_type': 'IPProtocolNSService',
                                 'protocol_number': 4}}],
                             '_revision': revision,
                             'disabled': False,
                             'sources': [{'target_id': target_id,
                                          'target_type': 'NSGroup'}],
                             'destinations': [{'target_id': group_id,
                                               'target_type': 'NSGroup'}],
                             'logged': False, 'action': 'ALLOW'}
            create.assert_called_once_with(
                'firewall/sections/%s/rules?action=create_multiple&'
                'operation=insert_bottom' % section_id,
                {'rules': [expected_rule]})


class TestNsxLibIPSet(nsxlib_testcase.NsxClientTestCase):
    """Tests for vmware_nsxlib.v3.security.NsxLibIPSet"""

    def test_get_ipset_reference(self):
        mock_ip_set = uuidutils.generate_uuid()
        result = self.nsxlib.ip_set.get_ipset_reference(
            mock_ip_set)
        expected = {
            'target_id': mock_ip_set,
            'target_type': const.IP_SET
        }
        self.assertEqual(expected, result)

    def test_create_ip_set(self):
        fake_ip_set = test_constants.FAKE_IP_SET.copy()
        data = {
            'display_name': fake_ip_set['display_name'],
            'ip_addresses': fake_ip_set['ip_addresses'],
            'description': 'ipset-desc',
            'tags': []
        }
        with mock.patch.object(self.nsxlib.client, 'create') as create:
            self.nsxlib.ip_set.create(
                fake_ip_set['display_name'], 'ipset-desc',
                ip_addresses=fake_ip_set['ip_addresses'])
            resource = 'ip-sets'
            create.assert_called_with(resource, data)

    def test_delete_ip_set(self):
        with mock.patch.object(self.nsxlib.client, 'delete') as delete:
            fake_ip_set = test_constants.FAKE_IP_SET.copy()
            self.nsxlib.ip_set.delete(fake_ip_set['id'])
            delete.assert_called_with('ip-sets/%s' % fake_ip_set['id'],
                                      headers=None)

    def test_update_ip_set(self):
        fake_ip_set = test_constants.FAKE_IP_SET.copy()
        new_ip_addresses = ['10.0.0.0']
        data = {
            'id': fake_ip_set['id'],
            'display_name': fake_ip_set['display_name'],
            'ip_addresses': new_ip_addresses,
            'resource_type': 'IPSet'
        }
        with mock.patch.object(self.nsxlib.client, 'get',
                               return_value=fake_ip_set):
            with mock.patch.object(self.nsxlib.client, 'update') as update:
                self.nsxlib.ip_set.update(
                    fake_ip_set['id'], ip_addresses=new_ip_addresses)
                resource = 'ip-sets/%s' % fake_ip_set['id']
                update.assert_called_with(resource, data, headers=None)

    def test_update_ip_set_empty_ip_addresses(self):
        fake_ip_set = test_constants.FAKE_IP_SET.copy()
        new_ip_addresses = []
        data = {
            'id': fake_ip_set['id'],
            'display_name': fake_ip_set['display_name'],
            'ip_addresses': new_ip_addresses,
            'resource_type': 'IPSet'
        }
        with mock.patch.object(self.nsxlib.client, 'get',
                               return_value=fake_ip_set):
            with mock.patch.object(self.nsxlib.client, 'update') as update:
                self.nsxlib.ip_set.update(
                    fake_ip_set['id'], ip_addresses=new_ip_addresses)
                resource = 'ip-sets/%s' % fake_ip_set['id']
                update.assert_called_with(resource, data, headers=None)

    def test_update_ip_set_callback(self):

        def update_payload_cbk(revised_payload, payload):
            payload['ip_addresses'] = (revised_payload['ip_addresses'] +
                                       payload['ip_addresses'])

        fake_ip_set = test_constants.FAKE_IP_SET.copy()
        new_ip_addresses = ['10.0.0.0']
        updated_ip_addresses = fake_ip_set['ip_addresses'] + new_ip_addresses
        data = {
            'id': fake_ip_set['id'],
            'display_name': fake_ip_set['display_name'],
            'ip_addresses': updated_ip_addresses,
            'resource_type': 'IPSet'
        }
        with mock.patch.object(self.nsxlib.client, 'get',
                               return_value=fake_ip_set):
            with mock.patch.object(self.nsxlib.client, 'update') as update:
                self.nsxlib.ip_set.update(
                    fake_ip_set['id'], ip_addresses=new_ip_addresses,
                    update_payload_cbk=update_payload_cbk)
                resource = 'ip-sets/%s' % fake_ip_set['id']
                update.assert_called_with(resource, data, headers=None)


class TestNsxLibNSGroup(nsxlib_testcase.NsxClientTestCase):
    """Tests for vmware_nsxlib.v3.security.NsxLibNSGroup"""

    def setUp(self, *args, **kwargs):
        super(TestNsxLibNSGroup, self).setUp(with_mocks=False)

    def test_get_nsgroup_lp_complex_expression(self):
        port_tags = {'app': 'foo', 'project': 'myproject'}
        port_exp = [self.nsxlib.ns_group.get_port_tag_expression(k, v)
                    for k, v in port_tags.items()]
        complex_exp = self.nsxlib.ns_group.get_nsgroup_complex_expression(
            expressions=port_exp)
        expected_exp = {'resource_type': const.NSGROUP_COMPLEX_EXP,
                        'expressions': port_exp}
        self.assertEqual(expected_exp, complex_exp)

    def test_get_nsgroup_ls_complex_expression(self):
        switch_tags = {'app': 'foo', 'project': 'myproject'}
        switch_exp = [self.nsxlib.ns_group.get_switch_tag_expression(k, v)
                      for k, v in switch_tags.items()]
        complex_exp = self.nsxlib.ns_group.get_nsgroup_complex_expression(
            expressions=switch_exp)
        expected_exp = {'resource_type': const.NSGROUP_COMPLEX_EXP,
                        'expressions': switch_exp}
        self.assertEqual(expected_exp, complex_exp)

    def test_update(self):
        nsg_tags = [{"scope": "name", "tag": "new_name"}]
        membership_criteria = []
        with mock.patch.object(self.nsxlib.client, 'update') as update:
            with mock.patch.object(self.nsxlib.client, 'get') as get:
                get.return_value = {}
                self.nsxlib.ns_group.update(
                    'nsgroupid', tags_update=nsg_tags,
                    membership_criteria=membership_criteria)
                resource = 'ns-groups/nsgroupid'
                data = {'tags': nsg_tags,
                        'membership_criteria': membership_criteria}
                update.assert_called_with(resource, data, headers=None)

    def test_list(self):
        ns_group = {'id': 'dummy'}
        ret_list = {'results': [ns_group]}
        with mock.patch.object(self.nsxlib.ns_group.client, 'list',
                               return_value=ret_list) as list_mock:
            result = self.nsxlib.ns_group.list()
            list_mock.assert_called_with('ns-groups?populate_references=false')
            self.assertEqual([ns_group], result)

    def test_find(self):
        name = 'test_name'
        ns_group = {'id': 'dummy', 'display_name': name}
        ret_list = {'results': [ns_group]}
        with mock.patch.object(self.nsxlib.ns_group.client, 'list',
                               return_value=ret_list):
            result = self.nsxlib.ns_group.find_by_display_name(name)
            self.assertEqual([ns_group], result)

    def test_read(self):
        ns_group_id = 'dummy'
        with mock.patch.object(self.nsxlib.ns_group.client, 'get') as get_mock:
            self.nsxlib.ns_group.read(ns_group_id)
            get_mock.assert_called_with(
                'ns-groups/%s?populate_references=true' % ns_group_id)

    def test_create(self):
        name = 'test_name'
        description = 'dummy'
        with mock.patch.object(self.nsxlib.ns_group.client,
                               'create') as create_mock:
            self.nsxlib.ns_group.create(name, description, None)
            create_mock.assert_called_with(
                'ns-groups',
                {'display_name': name,
                 'description': description,
                 'tags': None,
                 'members': []})

    def test_create_with_membership(self):
        name = 'test_name'
        description = 'dummy'
        membership = 'criteria'
        with mock.patch.object(self.nsxlib.ns_group.client,
                               'create') as create_mock:
            self.nsxlib.ns_group.create(name, description, None,
                                        membership_criteria=membership)
            create_mock.assert_called_with(
                'ns-groups',
                {'display_name': name,
                 'description': description,
                 'tags': None,
                 'members': [],
                 'membership_criteria': [membership]})

    def test_delete(self):
        ns_group_id = 'dummy'
        with mock.patch.object(self.nsxlib.ns_group.client,
                               'delete') as del_mock:
            self.nsxlib.ns_group.delete(ns_group_id)
            del_mock.assert_called_with(
                'ns-groups/%s?force=true' % ns_group_id, headers=None)

    def test_update_nsgroup_and_section(self):
        name = 'name'
        description = 'description'
        logging = False
        nsgroup_id = uuidutils.generate_uuid()
        section_id = uuidutils.generate_uuid()
        log_sg_allowed_traffic = True

        with mock.patch.object(self.nsxlib.client, 'update') as update_mock,\
            mock.patch.object(self.nsxlib.client, 'get') as get_mock:
            self.nsxlib.ns_group.update_nsgroup_and_section(
                name, description, logging, nsgroup_id, section_id,
                log_sg_allowed_traffic)
            # updating the nsgroup and the section
            self.assertEqual(2, update_mock.call_count)
            # getting the rules, and get before each update
            self.assertEqual(3, get_mock.call_count)

    def test_update_lport_nsgroups_with_mocks(self):
        nsgroup_id1 = uuidutils.generate_uuid()
        nsgroup_id2 = uuidutils.generate_uuid()
        lport_id = uuidutils.generate_uuid()
        original_nsgroups = [nsgroup_id1]
        updated_nsgroups = [nsgroup_id2]

        with mock.patch('vmware_nsxlib.v3.security.NsxLibNsGroup.'
                        'remove_member') as remove_mock,\
            mock.patch('vmware_nsxlib.v3.security.NsxLibNsGroup.'
                       'add_members') as add_mock:
            self.nsxlib.ns_group.update_lport_nsgroups(
                lport_id, original_nsgroups, updated_nsgroups)
            add_mock.assert_called_once_with(nsgroup_id2, 'LogicalPort',
                                             [lport_id])
            remove_mock.assert_called_once_with(nsgroup_id1, 'LogicalPort',
                                                lport_id)

    def test_update_lport_nsgroups(self):
        nsgroup_id1 = uuidutils.generate_uuid()
        nsgroup_id2 = uuidutils.generate_uuid()
        lport_id = uuidutils.generate_uuid()
        original_nsgroups = [nsgroup_id1]
        updated_nsgroups = [nsgroup_id2]

        with mock.patch.object(self.nsxlib.client, 'create') as update_mock:
            self.nsxlib.ns_group.update_lport_nsgroups(
                lport_id, original_nsgroups, updated_nsgroups)
            self.assertEqual(2, update_mock.call_count)



# Copyright 2021 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from unittest import mock

from vmware_nsxlib.tests.unit.v3.policy import test_resources
from vmware_nsxlib.v3.policy import core_defs


class TestAlbAuthTokenProvider(test_resources.NsxPolicyLibTestCase):

    def test_get_avi_lb_auth_token(self):
        avi_api = self.policy_lib.alb_token_provider
        with mock.patch.object(self.policy_lib.client, 'update') as update:
            avi_api.get_avi_lb_auth_token('avi_user')
            update.assert_called_with('infra/alb-auth-token',
                                      {'username': 'avi_user', 'hours': 1})

    def test_get_avi_endpoint_info(self):
        avi_api = self.policy_lib.alb_token_provider
        with mock.patch.object(self.policy_lib.client, 'get') as get:
            avi_api.get_avi_endpoint_info()
            get.assert_called_with(
                (core_defs.AVI_ENDPOINT_PATTERN % 'infra'))



# Copyright 2016 OpenStack Foundation
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from distutils import version

from oslo_log import log

from vmware_nsxlib._i18n import _
from vmware_nsxlib.v3 import client
from vmware_nsxlib.v3 import cluster_management
from vmware_nsxlib.v3 import core_resources
from vmware_nsxlib.v3 import exceptions
from vmware_nsxlib.v3 import lib
from vmware_nsxlib.v3 import load_balancer
from vmware_nsxlib.v3 import native_dhcp
from vmware_nsxlib.v3 import nsx_constants
from vmware_nsxlib.v3 import resources
from vmware_nsxlib.v3 import router
from vmware_nsxlib.v3 import security
from vmware_nsxlib.v3 import trust_management
from vmware_nsxlib.v3 import utils
from vmware_nsxlib.v3 import vpn_ipsec

LOG = log.getLogger(__name__)


class NsxLib(lib.NsxLibBase):

    def init_api(self):
        self.port_mirror = core_resources.NsxLibPortMirror(
            self.client, self.nsxlib_config, nsxlib=self)
        self.bridge_endpoint = core_resources.NsxLibBridgeEndpoint(
            self.client, self.nsxlib_config, nsxlib=self)
        self.bridge_endpoint_profile = (
            core_resources.NsxLibBridgeEndpointProfile(
                self.client, self.nsxlib_config, nsxlib=self))
        self.logical_switch = core_resources.NsxLibLogicalSwitch(
            self.client, self.nsxlib_config, nsxlib=self)
        self.logical_router = core_resources.NsxLibLogicalRouter(
            self.client, self.nsxlib_config, nsxlib=self)
        self.switching_profile = core_resources.NsxLibSwitchingProfile(
            self.client, self.nsxlib_config, nsxlib=self)
        self.qos_switching_profile = core_resources.NsxLibQosSwitchingProfile(
            self.client, self.nsxlib_config, nsxlib=self)
        self.edge_cluster = core_resources.NsxLibEdgeCluster(
            self.client, self.nsxlib_config, nsxlib=self)
        self.bridge_cluster = core_resources.NsxLibBridgeCluster(
            self.client, self.nsxlib_config, nsxlib=self)
        self.transport_zone = core_resources.NsxLibTransportZone(
            self.client, self.nsxlib_config, nsxlib=self)
        self.transport_node = core_resources.NsxLibTransportNode(
            self.client, self.nsxlib_config, nsxlib=self)
        self.relay_service = core_resources.NsxLibDhcpRelayService(
            self.client, self.nsxlib_config, nsxlib=self)
        self.relay_profile = core_resources.NsxLibDhcpRelayProfile(
            self.client, self.nsxlib_config, nsxlib=self)
        self.native_dhcp_profile = core_resources.NsxLibDhcpProfile(
            self.client, self.nsxlib_config, nsxlib=self)
        self.native_md_proxy = core_resources.NsxLibMetadataProxy(
            self.client, self.nsxlib_config, nsxlib=self)
        self.firewall_section = security.NsxLibFirewallSection(
            self.client, self.nsxlib_config, nsxlib=self)
        self.ns_group = security.NsxLibNsGroup(
            self.client, self.nsxlib_config, self.firewall_section,
            nsxlib=self)
        self.native_dhcp = native_dhcp.NsxLibNativeDhcp(
            self.client, self.nsxlib_config, nsxlib=self)
        self.ip_block_subnet = core_resources.NsxLibIpBlockSubnet(
            self.client, self.nsxlib_config, nsxlib=self)
        self.ip_block = core_resources.NsxLibIpBlock(
            self.client, self.nsxlib_config, nsxlib=self)
        self.ip_set = security.NsxLibIPSet(
            self.client, self.nsxlib_config, nsxlib=self)
        self.logical_port = resources.LogicalPort(
            self.client, self.nsxlib_config, nsxlib=self)
        self.logical_router_port = resources.LogicalRouterPort(
            self.client, self.nsxlib_config, nsxlib=self)
        self.dhcp_server = resources.LogicalDhcpServer(
            self.client, self.nsxlib_config, nsxlib=self)
        self.ip_pool = resources.IpPool(
            self.client, self.nsxlib_config, nsxlib=self)
        self.load_balancer = load_balancer.LoadBalancer(
            self.client, self.nsxlib_config, nsxlib=self)
        self.trust_management = trust_management.NsxLibTrustManagement(
            self.client, self.nsxlib_config)
        self.router = router.RouterLib(
            self.logical_router, self.logical_router_port, self)
        self.virtual_machine = core_resources.NsxLibFabricVirtualMachine(
            self.client, self.nsxlib_config, nsxlib=self)
        self.vif = core_resources.NsxLibFabricVirtualInterface(
            self.client, self.nsxlib_config, nsxlib=self)
        self.vpn_ipsec = vpn_ipsec.VpnIpSec(
            self.client, self.nsxlib_config, nsxlib=self)
        self.http_services = resources.NodeHttpServiceProperties(
            self.client, self.nsxlib_config, nsxlib=self)
        self.cluster_nodes = resources.NsxlibClusterNodesConfig(
            self.client, self.nsxlib_config, nsxlib=self)
        self.global_routing = core_resources.NsxLibGlobalRoutingConfig(
            self.client, self.nsxlib_config, nsxlib=self)
        self.host_switch_profiles = resources.NsxlibHostSwitchProfiles(
            self.client, self.nsxlib_config, nsxlib=self)
        self.cluster_management = cluster_management.NsxLibClusterManagement(
            self.client, self.nsxlib_config)

        # Update tag limits
        self.tag_limits = self.get_tag_limits()
        utils.update_tag_limits(self.tag_limits)

    @property
    def validate_connection_method(self):
        """Return a method that will validate the NSX manager status"""

        def check_manager_status(client, manager_url):
            """MP healthcheck for Version 2.4 and above"""
            # Try to get the status silently and with no retries
            status = client.get('reverse-proxy/node/health',
                                silent=True, with_retries=False)
            if not status or not status.get('healthy', False):
                msg = _("Manager is not in working state: %s") % status
                LOG.warning(msg)
                raise exceptions.ResourceNotFound(
                    manager=manager_url, operation=msg)

        return check_manager_status

    def get_version(self):
        if self.nsx_version:
            return self.nsx_version

        node = self.client.get("node")
        self.nsx_version = node.get('node_version')
        return self.nsx_version

    def export_restricted(self):
        node = self.client.get("node")
        return node.get('export_type') == 'RESTRICTED'

    def feature_supported(self, feature):
        if (version.LooseVersion(self.get_version()) >=
            version.LooseVersion(nsx_constants.NSX_VERSION_3_1_0)):
            # features available since 3.1.0
            if feature == nsx_constants.FEATURE_MP2P_MIGRATION:
                return True
            if feature == nsx_constants.FEATURE_SPOOFGUARD_CIDR:
                return True

        if (version.LooseVersion(self.get_version()) >=
            version.LooseVersion(nsx_constants.NSX_VERSION_3_0_0)):
            # features available since 3.0.0
            if (feature == nsx_constants.FEATURE_GET_TZ_FROM_SWITCH):
                return True
            if (feature == nsx_constants.FEATURE_RELAX_SCALE_VALIDATION):
                return True

        if (version.LooseVersion(self.get_version()) >=
            version.LooseVersion(nsx_constants.NSX_VERSION_2_5_0)):
            # features available since 2.5
            if (feature == nsx_constants.FEATURE_CONTAINER_CLUSTER_INVENTORY):
                return True
            if (feature == nsx_constants.FEATURE_IPV6):
                return True
            if (feature == nsx_constants.FEATURE_ENS_WITH_QOS):
                return True

        if (version.LooseVersion(self.get_version()) >=
            version.LooseVersion(nsx_constants.NSX_VERSION_2_4_0)):
            # Features available since 2.4
            if (feature == nsx_constants.FEATURE_ENS_WITH_SEC):
                return True
            if (feature == nsx_constants.FEATURE_ICMP_STRICT):
                return True
            if (feature == nsx_constants.FEATURE_ENABLE_STANDBY_RELOCATION):
                return True

        if (version.LooseVersion(self.get_version()) >=
            version.LooseVersion(nsx_constants.NSX_VERSION_2_3_0)):
            # Features available since 2.3
            if (feature == nsx_constants.FEATURE_ROUTER_ALLOCATION_PROFILE):
                return True
            if (feature == nsx_constants.FEATURE_LB_HM_RESPONSE_CODES):
                return True

        if (version.LooseVersion(self.get_version()) >=
            version.LooseVersion(nsx_constants.NSX_VERSION_2_2_0)):
            # Features available since 2.2
            if (feature == nsx_constants.FEATURE_VLAN_ROUTER_INTERFACE or
                feature == nsx_constants.FEATURE_IPSEC_VPN or
                feature == nsx_constants.FEATURE_ON_BEHALF_OF or
                feature == nsx_constants.FEATURE_RATE_LIMIT or
                feature == nsx_constants.FEATURE_TRUNK_VLAN or
                feature == nsx_constants.FEATURE_ROUTER_TRANSPORT_ZONE or
                feature == nsx_constants.FEATURE_NO_DNAT_NO_SNAT):
                return True

        if (version.LooseVersion(self.get_version()) >=
            version.LooseVersion(nsx_constants.NSX_VERSION_2_1_0)):
            # Features available since 2.1
            if (feature == nsx_constants.FEATURE_LOAD_BALANCER):
                return True

        if (version.LooseVersion(self.get_version()) >=
            version.LooseVersion(nsx_constants.NSX_VERSION_2_0_0)):
            # Features available since 2.0
            if (feature == nsx_constants.FEATURE_EXCLUDE_PORT_BY_TAG or
                feature == nsx_constants.FEATURE_ROUTER_FIREWALL or
                feature == nsx_constants.FEATURE_DHCP_RELAY):
                return True

        if (version.LooseVersion(self.get_version()) >=
            version.LooseVersion(nsx_constants.NSX_VERSION_1_1_0)):
            # Features available since 1.1
            if (feature == nsx_constants.FEATURE_MAC_LEARNING or
                feature == nsx_constants.FEATURE_DYNAMIC_CRITERIA):
                return True

        return False

    @property
    def client_url_prefix(self):
        return client.NSX3Client.NSX_V1_API_PREFIX



# Copyright 2018 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import copy

from oslo_log import log as logging

from vmware_nsxlib._i18n import _
from vmware_nsxlib.v3 import exceptions as nsxlib_exc
from vmware_nsxlib.v3.policy import constants
from vmware_nsxlib.v3.policy.core_resources import IGNORE
from vmware_nsxlib.v3.policy.core_resources import NsxPolicyResourceBase
from vmware_nsxlib.v3.policy import lb_defs
from vmware_nsxlib.v3 import utils


LOG = logging.getLogger(__name__)


class NsxPolicyLBAppProfileBase(NsxPolicyResourceBase):
    """NSX Policy LB app profile"""

    def create_or_overwrite(self, name,
                            lb_app_profile_id=None,
                            description=IGNORE,
                            http_redirect_to_https=IGNORE,
                            http_redirect_to=IGNORE,
                            idle_timeout=IGNORE,
                            ntlm=IGNORE,
                            request_body_size=IGNORE,
                            request_header_size=IGNORE,
                            response_header_size=IGNORE,
                            response_timeout=IGNORE,
                            x_forwarded_for=IGNORE,
                            tags=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):
        lb_app_profile_id = self._init_obj_uuid(lb_app_profile_id)
        lb_app_profile_def = self._init_def(
            lb_app_profile_id=lb_app_profile_id,
            name=name,
            description=description,
            http_redirect_to_https=http_redirect_to_https,
            http_redirect_to=http_redirect_to,
            idle_timeout=idle_timeout,
            ntlm=ntlm,
            request_body_size=request_body_size,
            request_header_size=request_header_size,
            response_header_size=response_header_size,
            response_timeout=response_timeout,
            x_forwarded_for=x_forwarded_for,
            tags=tags,
            tenant=tenant)
        self._create_or_store(lb_app_profile_def)
        return lb_app_profile_id

    def delete(self, lb_app_profile_id,
               tenant=constants.POLICY_INFRA_TENANT):
        lb_app_profile_def = self.entry_def(
            lb_app_profile_id=lb_app_profile_id,
            tenant=tenant)
        self._delete_with_retry(lb_app_profile_def)

    def get(self, lb_app_profile_id,
            tenant=constants.POLICY_INFRA_TENANT, silent=False):
        lb_app_profile_def = self.entry_def(
            lb_app_profile_id=lb_app_profile_id,
            tenant=tenant)
        return self.policy_api.get(lb_app_profile_def, silent=silent)

    def list(self, tenant=constants.POLICY_INFRA_TENANT):
        lb_app_profile_def = self.entry_def(tenant=tenant)
        return self._list(lb_app_profile_def)

    def update(self, lb_app_profile_id,
               name=IGNORE,
               description=IGNORE,
               http_redirect_to_https=IGNORE,
               http_redirect_to=IGNORE,
               idle_timeout=IGNORE,
               ntlm=IGNORE,
               request_body_size=IGNORE,
               request_header_size=IGNORE,
               response_header_size=IGNORE,
               response_timeout=IGNORE,
               x_forwarded_for=IGNORE,
               tags=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT):
        self._update(
            lb_app_profile_id=lb_app_profile_id,
            name=name,
            description=description,
            http_redirect_to_https=http_redirect_to_https,
            http_redirect_to=http_redirect_to,
            idle_timeout=idle_timeout,
            ntlm=ntlm,
            request_body_size=request_body_size,
            request_header_size=request_header_size,
            response_header_size=response_header_size,
            response_timeout=response_timeout,
            x_forwarded_for=x_forwarded_for,
            tags=tags,
            tenant=tenant)

    def get_path(self, lb_app_profile_id,
                 tenant=constants.POLICY_INFRA_TENANT):
        profile_def = self.entry_def(
            lb_app_profile_id=lb_app_profile_id,
            tenant=tenant)
        return profile_def.get_resource_full_path()


class NsxPolicyLBAppProfileHttpApi(NsxPolicyLBAppProfileBase):
    """NSX Policy LB app profile"""

    @property
    def entry_def(self):
        return lb_defs.LBHttpProfileDef


class NsxPolicyLBAppProfileFastTcpApi(
    NsxPolicyLBAppProfileBase):
    """NSX Policy LB app profile"""

    @property
    def entry_def(self):
        return lb_defs.LBFastTcpProfile


class NsxPolicyLBAppProfileFastUdpApi(
    NsxPolicyLBAppProfileBase):
    """NSX Policy LB app profile"""

    @property
    def entry_def(self):
        return lb_defs.LBFastUdpProfile


class NsxPolicyLoadBalancerServerSSLProfileApi(NsxPolicyResourceBase):
    """NSX Policy LB server ssl profile"""

    @property
    def entry_def(self):
        return lb_defs.LBServerSslProfileDef

    def create_or_overwrite(self, name, server_ssl_profile_id=None,
                            description=IGNORE, tags=IGNORE,
                            cipher_group_label=IGNORE, ciphers=IGNORE,
                            protocols=IGNORE, session_cache_enabled=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):
        server_ssl_profile_id = self._init_obj_uuid(server_ssl_profile_id)
        lb_server_ssl_profile_def = self._init_def(
            server_ssl_profile_id=server_ssl_profile_id,
            name=name,
            description=description,
            tags=tags,
            protocols=protocols,
            tenant=tenant)
        self._create_or_store(lb_server_ssl_profile_def)
        return server_ssl_profile_id

    def delete(self, server_ssl_profile_id,
               tenant=constants.POLICY_INFRA_TENANT):
        lb_server_ssl_profile_def = self.entry_def(
            server_ssl_profile_id=server_ssl_profile_id,
            tenant=tenant)
        self._delete_with_retry(lb_server_ssl_profile_def)

    def get(self, server_ssl_profile_id,
            tenant=constants.POLICY_INFRA_TENANT, silent=False):
        lb_server_ssl_profile_def = self.entry_def(
            server_ssl_profile_id=server_ssl_profile_id,
            tenant=tenant)
        return self.policy_api.get(lb_server_ssl_profile_def, silent=silent)

    def list(self, tenant=constants.POLICY_INFRA_TENANT):
        lb_server_ssl_profile_def = self.entry_def(tenant=tenant)
        return self._list(lb_server_ssl_profile_def)

    def update(self, server_ssl_profile_id,
               name=IGNORE, description=IGNORE, tags=IGNORE,
               cipher_group_label=IGNORE, ciphers=IGNORE,
               protocols=IGNORE, session_cache_enabled=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT):
        self._update(
            server_ssl_profile_id=server_ssl_profile_id,
            name=name,
            description=description,
            tags=tags,
            protocols=protocols,
            tenant=tenant)


class NsxPolicyLoadBalancerClientSSLProfileApi(NsxPolicyResourceBase):
    """NSX Policy LB client ssl profile"""

    @property
    def entry_def(self):
        return lb_defs.LBClientSslProfileDef

    def create_or_overwrite(self, name,
                            client_ssl_profile_id=None,
                            description=IGNORE,
                            tags=IGNORE,
                            protocols=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):
        client_ssl_profile_id = self._init_obj_uuid(
            client_ssl_profile_id)
        lb_client_ssl_profile_def = self._init_def(
            client_ssl_profile_id=client_ssl_profile_id,
            name=name,
            description=description,
            tags=tags,
            protocols=protocols,
            tenant=tenant)
        self._create_or_store(lb_client_ssl_profile_def)
        return client_ssl_profile_id

    def delete(self, client_ssl_profile_id,
               tenant=constants.POLICY_INFRA_TENANT):
        lb_client_ssl_profile_def = self.entry_def(
            client_ssl_profile_id=client_ssl_profile_id,
            tenant=tenant)
        self._delete_with_retry(lb_client_ssl_profile_def)

    def get(self, client_ssl_profile_id,
            tenant=constants.POLICY_INFRA_TENANT, silent=False):
        lb_client_ssl_profile_def = self.entry_def(
            client_ssl_profile_id=client_ssl_profile_id,
            tenant=tenant)
        return self.policy_api.get(lb_client_ssl_profile_def, silent=silent)

    def list(self, tenant=constants.POLICY_INFRA_TENANT):
        lb_client_ssl_profile_def = self.entry_def(tenant=tenant)
        return self._list(lb_client_ssl_profile_def)

    def update(self, client_ssl_profile_id,
               name=IGNORE,
               description=IGNORE,
               tags=IGNORE,
               protocols=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT):
        self._update(
            client_ssl_profile_id=client_ssl_profile_id,
            name=name,
            description=description,
            tags=tags,
            protocols=protocols,
            tenant=tenant)

    def get_path(self, profile_id,
                 tenant=constants.POLICY_INFRA_TENANT):
        profile_def = self.entry_def(
            client_ssl_profile_id=profile_id,
            tenant=tenant)
        return profile_def.get_resource_full_path()


class NsxPolicyLoadBalancerPersistenceProfileApi(
    NsxPolicyResourceBase):
    """LB generic api for all types of session persistence profiles"""

    @property
    def entry_def(self):
        return lb_defs.LBPersistenceProfileBase

    def create_or_overwrite(self, name,
                            persistence_profile_id=None,
                            description=IGNORE,
                            tags=IGNORE,
                            ha_persistence_mirroring_enabled=IGNORE,
                            persistence_shared=IGNORE,
                            purge=IGNORE,
                            timeout=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):
        raise nsxlib_exc.NotImplemented(
            "Creating generic persistence profile")

    def delete(self, persistence_profile_id,
               tenant=constants.POLICY_INFRA_TENANT):
        persistence_profile_def = self.entry_def(
            persistence_profile_id=persistence_profile_id,
            tenant=tenant)
        self._delete_with_retry(persistence_profile_def)

    def get(self, persistence_profile_id,
            tenant=constants.POLICY_INFRA_TENANT, silent=False):
        persistence_profile_def = self.entry_def(
            persistence_profile_id=persistence_profile_id,
            tenant=tenant)
        return self.policy_api.get(persistence_profile_def, silent=silent)

    def list(self, tenant=constants.POLICY_INFRA_TENANT):
        persistence_profile_def = self.entry_def(tenant=tenant)
        return self._list(persistence_profile_def)

    def update(self, persistence_profile_id,
               name=IGNORE,
               description=IGNORE,
               tags=IGNORE,
               ha_persistence_mirroring_enabled=IGNORE,
               persistence_shared=IGNORE,
               purge=IGNORE,
               timeout=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT):
        raise nsxlib_exc.NotImplemented(
            "Updating generic persistence profile")

    def get_path(self, profile_id,
                 tenant=constants.POLICY_INFRA_TENANT):
        profile_def = self.entry_def(
            persistence_profile_id=profile_id,
            tenant=tenant)
        return profile_def.get_resource_full_path()

    def wait_until_realized(self, pers_id,
                            entity_type='LbPersistenceProfileDto',
                            tenant=constants.POLICY_INFRA_TENANT,
                            sleep=None, max_attempts=None):
        pers_def = self.entry_def(
            persistence_profile_id=pers_id, tenant=tenant)
        return self._wait_until_realized(
            pers_def, entity_type=entity_type,
            sleep=sleep, max_attempts=max_attempts)


class NsxPolicyLoadBalancerCookiePersistenceProfileApi(
    NsxPolicyLoadBalancerPersistenceProfileApi):
    """NSX Policy LB cookie persistence profile"""

    @property
    def entry_def(self):
        return lb_defs.LBCookiePersistenceProfileDef

    def create_or_overwrite(self, name,
                            persistence_profile_id=None,
                            description=IGNORE,
                            tags=IGNORE,
                            cookie_garble=IGNORE,
                            cookie_name=IGNORE,
                            cookie_mode=IGNORE,
                            cookie_path=IGNORE,
                            cookie_time=IGNORE,
                            persistence_shared=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):
        persistence_profile_id = self._init_obj_uuid(
            persistence_profile_id)
        lb_cookie_persistence_profile_def = self._init_def(
            persistence_profile_id=persistence_profile_id,
            name=name,
            description=description,
            tags=tags,
            cookie_name=cookie_name,
            cookie_garble=cookie_garble,
            cookie_mode=cookie_mode,
            cookie_path=cookie_path,
            cookie_time=cookie_time,
            persistence_shared=persistence_shared,
            tenant=tenant)
        self._create_or_store(lb_cookie_persistence_profile_def)
        return persistence_profile_id

    def list(self, tenant=constants.POLICY_INFRA_TENANT):
        lb_cookie_persistence_profile_def = self.entry_def(tenant=tenant)
        results = self._list(lb_cookie_persistence_profile_def)
        # filter the results by resource type
        return [res for res in results
                if res.get('resource_type') == self.entry_def.resource_type()]

    def update(self, persistence_profile_id,
               name=IGNORE,
               description=IGNORE,
               tags=IGNORE,
               cookie_garble=IGNORE,
               cookie_name=IGNORE,
               cookie_mode=IGNORE,
               cookie_path=IGNORE,
               cookie_time=IGNORE,
               persistence_shared=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT):
        self._update(
            persistence_profile_id=persistence_profile_id,
            name=name,
            description=description,
            tags=tags,
            cookie_garble=cookie_garble,
            cookie_mode=cookie_mode,
            cookie_name=cookie_name,
            cookie_path=cookie_path,
            cookie_time=cookie_time,
            persistence_shared=persistence_shared,
            tenant=tenant)


class NsxPolicyLoadBalancerSourceIpPersistenceProfileApi(
    NsxPolicyLoadBalancerPersistenceProfileApi):
    """NSX Policy LB source ip persistence profile"""

    @property
    def entry_def(self):
        return lb_defs.LBSourceIpPersistenceProfileDef

    def create_or_overwrite(self, name,
                            persistence_profile_id=None,
                            description=IGNORE,
                            tags=IGNORE,
                            ha_persistence_mirroring_enabled=IGNORE,
                            persistence_shared=IGNORE,
                            purge=IGNORE,
                            timeout=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):
        persistence_profile_id = self._init_obj_uuid(
            persistence_profile_id)
        lb_source_ip_persistence_profile_def = self._init_def(
            persistence_profile_id=persistence_profile_id,
            name=name,
            description=description,
            tags=tags,
            ha_persistence_mirroring_enabled=ha_persistence_mirroring_enabled,
            persistence_shared=persistence_shared,
            purge=purge,
            timeout=timeout,
            tenant=tenant)
        self._create_or_store(lb_source_ip_persistence_profile_def)
        return persistence_profile_id

    def list(self, tenant=constants.POLICY_INFRA_TENANT):
        lb_source_ip_persistence_profile_def = self.entry_def(tenant=tenant)
        results = self._list(lb_source_ip_persistence_profile_def)
        # filter the results by resource type
        return [res for res in results
                if res.get('resource_type') == self.entry_def.resource_type()]

    def update(self, persistence_profile_id,
               resource_type=IGNORE,
               name=IGNORE,
               description=IGNORE,
               tags=IGNORE,
               ha_persistence_mirroring_enabled=IGNORE,
               persistence_shared=IGNORE,
               purge=IGNORE,
               timeout=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT):
        self._update(
            persistence_profile_id=persistence_profile_id,
            resource_type=resource_type,
            name=name,
            description=description,
            tags=tags,
            ha_persistence_mirroring_enabled=ha_persistence_mirroring_enabled,
            persistence_shared=persistence_shared,
            purge=purge,
            timeout=timeout,
            tenant=tenant)


class NsxPolicyLoadBalancerPoolApi(NsxPolicyResourceBase):
    """NSX Policy LBService."""
    @property
    def entry_def(self):
        return lb_defs.LBPoolDef

    def create_or_overwrite(self, name, lb_pool_id=None, description=IGNORE,
                            tags=IGNORE, members=IGNORE, algorithm=IGNORE,
                            active_monitor_paths=IGNORE, member_group=IGNORE,
                            snat_translation=IGNORE,
                            tcp_multiplexing_enabled=IGNORE,
                            tcp_multiplexing_number=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):
        lb_pool_id = self._init_obj_uuid(lb_pool_id)
        lb_pool_def = self._init_def(
            lb_pool_id=lb_pool_id,
            name=name,
            description=description,
            tags=tags,
            members=members,
            active_monitor_paths=active_monitor_paths,
            algorithm=algorithm,
            member_group=member_group,
            snat_translation=snat_translation,
            tcp_multiplexing_enabled=tcp_multiplexing_enabled,
            tcp_multiplexing_number=tcp_multiplexing_number,
            tenant=tenant)

        self._create_or_store(lb_pool_def)
        return lb_pool_id

    def delete(self, lb_pool_id,
               tenant=constants.POLICY_INFRA_TENANT):
        lb_pool_def = self.entry_def(
            lb_pool_id=lb_pool_id, tenant=tenant)
        self._delete_or_store(lb_pool_def)

    def get(self, lb_pool_id, tenant=constants.POLICY_INFRA_TENANT,
            silent=False):
        lb_pool_def = self.entry_def(
            lb_pool_id=lb_pool_id, tenant=tenant)
        return self.policy_api.get(lb_pool_def, silent=silent)

    def list(self, tenant=constants.POLICY_INFRA_TENANT):
        lb_pool_def = self.entry_def(tenant=tenant)
        return self.policy_api.list(lb_pool_def)['results']

    def update(self, lb_pool_id, name=IGNORE, description=IGNORE,
               tags=IGNORE, members=IGNORE, algorithm=IGNORE,
               active_monitor_paths=IGNORE, member_group=IGNORE,
               snat_translation=IGNORE, tcp_multiplexing_enabled=IGNORE,
               tcp_multiplexing_number=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT,
               allow_partial_updates=True):
        self._update(
            lb_pool_id=lb_pool_id,
            name=name,
            description=description,
            tags=tags,
            members=members,
            active_monitor_paths=active_monitor_paths,
            algorithm=algorithm,
            member_group=member_group,
            snat_translation=snat_translation,
            tcp_multiplexing_enabled=tcp_multiplexing_enabled,
            tcp_multiplexing_number=tcp_multiplexing_number,
            tenant=tenant, allow_partial_updates=allow_partial_updates)

    def add_monitor_to_pool(self, lb_pool_id, active_monitor_paths,
                            tenant=constants.POLICY_INFRA_TENANT):
        lb_pool_def = self.entry_def(
            lb_pool_id=lb_pool_id, tenant=tenant)
        lb_pool = self.policy_api.get(lb_pool_def)
        monitor_paths = lb_pool.get('active_monitor_paths', [])
        monitor_paths.extend(active_monitor_paths)
        self._update(
            lb_pool_id=lb_pool_id, active_monitor_paths=monitor_paths,
            tenant=tenant)

    def remove_monitor_from_pool(self, lb_pool_id, monitor_path,
                                 tenant=constants.POLICY_INFRA_TENANT):
        lb_pool_def = self.entry_def(
            lb_pool_id=lb_pool_id, tenant=tenant)
        lb_pool = self.policy_api.get(lb_pool_def)
        monitor_paths = lb_pool.get('active_monitor_paths', [])
        if monitor_path in monitor_paths:
            monitor_paths.remove(monitor_path)
            self._update(lb_pool_id=lb_pool_id,
                         active_monitor_paths=monitor_paths,
                         tenant=tenant)

    def create_pool_member_and_add_to_pool(
            self, lb_pool_id, ip_address, port=None,
            display_name=None, weight=None, admin_state=None,
            backup_member=None,
            tenant=constants.POLICY_INFRA_TENANT):
        lb_pool_member = lb_defs.LBPoolMemberDef(
            ip_address, port=port,
            name=display_name,
            weight=weight, admin_state=admin_state,
            backup_member=backup_member)
        lb_pool_def = lb_defs.LBPoolDef(
            lb_pool_id=lb_pool_id, tenant=tenant)
        lb_pool = self.policy_api.get(lb_pool_def)
        lb_pool_members = lb_pool.get('members', [])
        lb_pool_members.append(lb_pool_member)
        self._update(lb_pool_id=lb_pool_id, members=lb_pool_members,
                     tenant=tenant)
        return lb_pool_member

    def update_pool_member(
            self, lb_pool_id, ip_address, port=None,
            display_name=None, weight=None, admin_state=None,
            backup_member=None,
            tenant=constants.POLICY_INFRA_TENANT):
        lb_pool_def = lb_defs.LBPoolDef(
            lb_pool_id=lb_pool_id, tenant=tenant)
        lb_pool = self.policy_api.get(lb_pool_def)
        lb_pool_members = lb_pool.get('members', [])
        member_to_update = [x for x in lb_pool_members if (
            x.get('ip_address') == ip_address and x.get('port') == str(port))]
        if member_to_update:
            if display_name:
                member_to_update[0]['display_name'] = display_name
            if weight:
                member_to_update[0]['weight'] = weight
            if admin_state:
                member_to_update[0]['admin_state'] = admin_state
            if backup_member is not None:
                member_to_update[0]['backup_member'] = backup_member
            self._update(lb_pool_id=lb_pool_id, members=lb_pool_members,
                         tenant=tenant)
        else:
            ops = ('Updating member %(address)s:%(port)d failed, not found in '
                   'pool %(pool)s', {'address': ip_address,
                                     'port': port,
                                     'pool': lb_pool_id})
            raise nsxlib_exc.ResourceNotFound(manager=lb_pool_def,
                                              operation=ops)

    def remove_pool_member(self, lb_pool_id, ip_address, port=None,
                           tenant=constants.POLICY_INFRA_TENANT):
        lb_pool_def = lb_defs.LBPoolDef(
            lb_pool_id=lb_pool_id, tenant=tenant)
        lb_pool = self.policy_api.get(lb_pool_def)
        lb_pool_members = lb_pool.get('members', [])
        lb_pool_members = [x for x in lb_pool_members if (
            x.get('ip_address') != ip_address or x.get('port') != str(port))]
        self._update(lb_pool_id=lb_pool_id, members=lb_pool_members,
                     tenant=tenant)

    def get_path(self, lb_pool_id,
                 tenant=constants.POLICY_INFRA_TENANT):
        profile_def = self.entry_def(
            lb_pool_id=lb_pool_id,
            tenant=tenant)
        return profile_def.get_resource_full_path()

    def wait_until_realized(self, lb_pool_id, entity_type='LbPoolDto',
                            tenant=constants.POLICY_INFRA_TENANT,
                            sleep=None, max_attempts=None):
        lb_pool_def = self.entry_def(
            lb_pool_id=lb_pool_id, tenant=tenant)
        return self._wait_until_realized(
            lb_pool_def, entity_type=entity_type,
            sleep=sleep, max_attempts=max_attempts)


class NsxPolicyLoadBalancerServiceApi(NsxPolicyResourceBase):
    """NSX Policy LBService."""

    @property
    def entry_def(self):
        return lb_defs.LBServiceDef

    def create_or_overwrite(self, name, lb_service_id=None,
                            description=IGNORE,
                            tags=IGNORE,
                            size=IGNORE,
                            connectivity_path=IGNORE,
                            relax_scale_validation=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):
        lb_service_id = self._init_obj_uuid(lb_service_id)
        lb_service_def = self._init_def(
            lb_service_id=lb_service_id,
            name=name,
            description=description,
            tags=tags,
            size=size,
            connectivity_path=connectivity_path,
            relax_scale_validation=relax_scale_validation,
            tenant=tenant)

        self._create_or_store(lb_service_def)
        return lb_service_id

    def delete(self, lb_service_id,
               tenant=constants.POLICY_INFRA_TENANT):
        lb_service_def = self.entry_def(
            lb_service_id=lb_service_id, tenant=tenant)
        self._delete_with_retry(lb_service_def)

    def get(self, lb_service_id,
            tenant=constants.POLICY_INFRA_TENANT, silent=False):
        lb_service_def = self.entry_def(
            lb_service_id=lb_service_id, tenant=tenant)
        return self.policy_api.get(lb_service_def, silent=silent)

    def list(self, tenant=constants.POLICY_INFRA_TENANT, silent=False,
             silent_if_empty=False):
        if not silent and silent_if_empty:
            # Log only non-empty results
            silent = True
        lb_service_def = lb_defs.LBServiceDef(tenant=tenant)
        list_results = self.policy_api.list(lb_service_def, silent=silent)
        if silent_if_empty and list_results.get('results'):
            LOG.debug("REST call: GET %s. Response: %s",
                      lb_service_def.get_section_path(), list_results)

        return list_results['results']

    def update(self, lb_service_id, name=IGNORE,
               description=IGNORE, tags=IGNORE,
               size=IGNORE, connectivity_path=IGNORE,
               relax_scale_validation=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT):

        self._update(
            lb_service_id=lb_service_id,
            name=name,
            description=description,
            tags=tags,
            size=size,
            connectivity_path=connectivity_path,
            relax_scale_validation=relax_scale_validation,
            tenant=tenant)

    def update_customized(self, lb_service_id, update_payload_cbk,
                          tenant=constants.POLICY_INFRA_TENANT):
        """Update the LB service using GET & PUT

        Changing the body with a customized callback
        """

        lb_service_def = self.entry_def(
            lb_service_id=lb_service_id, tenant=tenant)
        lb_service_path = lb_service_def.get_resource_path()

        @utils.retry_upon_exception(
            nsxlib_exc.StaleRevision,
            max_attempts=self.policy_api.client.max_attempts)
        def _update():
            # Get the current data of service
            lb_service_body = self.policy_api.get(lb_service_def)
            # Update the body with the supplied callback
            update_payload_cbk(lb_service_body)
            # Update the backend using PUT
            self.policy_api.client.update(lb_service_path, lb_service_body)

        _update()

    def get_statistics(self, lb_service_id, realtime=False,
                       tenant=constants.POLICY_INFRA_TENANT,
                       silent=False):
        lb_service_stats_def = (
            lb_defs.LBServiceStatisticsDef(
                lb_service_id=lb_service_id,
                realtime=realtime,
                tenant=tenant))
        return self.policy_api.get(lb_service_stats_def, silent=silent)

    def get_status(self, lb_service_id,
                   tenant=constants.POLICY_INFRA_TENANT,
                   silent=False):
        lb_service_status_def = (
            lb_defs.LBServiceStatusDef(
                lb_service_id=lb_service_id,
                tenant=tenant))
        return self.policy_api.get(lb_service_status_def, silent=silent)

    def get_virtual_server_status(self, lb_service_id, lb_virtual_server_id,
                                  tenant=constants.POLICY_INFRA_TENANT):
        lb_vs_status_def = (
            lb_defs.LBVirtualServerStatusDef(
                lb_service_id=lb_service_id,
                lb_virtual_server_id=lb_virtual_server_id,
                tenant=tenant))
        return self.policy_api.get(lb_vs_status_def)

    def get_usage(self, lb_service_id, realtime=False,
                  tenant=constants.POLICY_INFRA_TENANT):
        lb_service_status_def = lb_defs.LBServiceUsageDef(
            lb_service_id=lb_service_id, realtime=realtime, tenant=tenant)
        return self.policy_api.get(lb_service_status_def)

    def get_path(self, lb_service_id,
                 tenant=constants.POLICY_INFRA_TENANT):
        profile_def = self.entry_def(
            lb_service_id=lb_service_id,
            tenant=tenant)
        return profile_def.get_resource_full_path()

    def wait_until_realized(self, lb_service_id, entity_type='LbServiceDto',
                            tenant=constants.POLICY_INFRA_TENANT,
                            sleep=None, max_attempts=None):
        lb_service_def = self.entry_def(
            lb_service_id=lb_service_id, tenant=tenant)
        return self._wait_until_realized(
            lb_service_def, entity_type=entity_type,
            sleep=sleep, max_attempts=max_attempts)


class NsxPolicyLoadBalancerVirtualServerAPI(NsxPolicyResourceBase):
    """NSX Policy LoadBalancerVirtualServers"""

    @property
    def entry_def(self):
        return lb_defs.LBVirtualServerDef

    def create_or_overwrite(self, name, virtual_server_id=None,
                            description=IGNORE,
                            rules=IGNORE, application_profile_id=IGNORE,
                            ip_address=IGNORE, lb_service_id=IGNORE,
                            client_ssl_profile_binding=IGNORE,
                            pool_id=IGNORE,
                            lb_persistence_profile_id=IGNORE,
                            ports=IGNORE,
                            server_ssl_profile_binding=IGNORE,
                            waf_profile_binding=IGNORE,
                            max_concurrent_connections=IGNORE,
                            access_list_control=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT,
                            tags=IGNORE, access_log_enabled=IGNORE,
                            log_significant_event_only=IGNORE):
        virtual_server_id = self._init_obj_uuid(virtual_server_id)
        lbvs_def = self._init_def(
            virtual_server_id=virtual_server_id,
            name=name,
            description=description,
            tenant=tenant,
            rules=rules,
            application_profile_id=application_profile_id,
            ip_address=ip_address,
            lb_service_id=lb_service_id,
            client_ssl_profile_binding=client_ssl_profile_binding,
            pool_id=pool_id,
            lb_persistence_profile_id=lb_persistence_profile_id,
            ports=ports,
            server_ssl_profile_binding=server_ssl_profile_binding,
            waf_profile_binding=waf_profile_binding,
            max_concurrent_connections=max_concurrent_connections,
            access_list_control=access_list_control,
            tags=tags,
            access_log_enabled=access_log_enabled,
            log_significant_event_only=log_significant_event_only
        )
        self._create_or_store(lbvs_def)
        return virtual_server_id

    def delete(self, virtual_server_id,
               tenant=constants.POLICY_INFRA_TENANT):
        lbvs_def = self.entry_def(
            virtual_server_id=virtual_server_id, tenant=tenant)
        self._delete_or_store(lbvs_def)

    def get(self, virtual_server_id,
            tenant=constants.POLICY_INFRA_TENANT, silent=False):
        lbvs_def = self.entry_def(
            virtual_server_id=virtual_server_id, tenant=tenant)
        return self.policy_api.get(lbvs_def, silent=silent)

    def list(self, tenant=constants.POLICY_INFRA_TENANT):
        lbvs_def = self.entry_def(tenant=tenant)
        return self.policy_api.list(lbvs_def)['results']

    def update(self, virtual_server_id, name=IGNORE, description=IGNORE,
               rules=IGNORE, application_profile_id=IGNORE,
               ip_address=IGNORE, lb_service_id=IGNORE,
               client_ssl_profile_binding=IGNORE,
               pool_id=IGNORE,
               lb_persistence_profile_id=IGNORE,
               ports=IGNORE,
               server_ssl_profile_binding=IGNORE,
               waf_profile_binding=IGNORE,
               max_concurrent_connections=IGNORE,
               access_list_control=IGNORE,
               tags=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT,
               allow_partial_updates=True, access_log_enabled=IGNORE,
               log_significant_event_only=IGNORE):

        @utils.retry_upon_exception(
            nsxlib_exc.StaleRevision,
            max_attempts=self.policy_api.client.max_attempts)
        def _update():
            self._update(
                virtual_server_id=virtual_server_id,
                name=name,
                description=description,
                tenant=tenant,
                rules=rules,
                application_profile_id=application_profile_id,
                ip_address=ip_address,
                lb_service_id=lb_service_id,
                client_ssl_profile_binding=client_ssl_profile_binding,
                pool_id=pool_id,
                lb_persistence_profile_id=lb_persistence_profile_id,
                ports=ports,
                server_ssl_profile_binding=server_ssl_profile_binding,
                waf_profile_binding=waf_profile_binding,
                max_concurrent_connections=max_concurrent_connections,
                access_list_control=access_list_control,
                tags=tags,
                allow_partial_updates=allow_partial_updates,
                access_log_enabled=access_log_enabled,
                log_significant_event_only=log_significant_event_only)

        _update()

    def update_virtual_server_with_pool(
            self, virtual_server_id, pool_id=IGNORE,
            tenant=constants.POLICY_INFRA_TENANT):
        return self.update(
            virtual_server_id, pool_id=pool_id, tenant=tenant)

    def update_virtual_server_application_profile(
            self, virtual_server_id, application_profile_id=IGNORE,
            tenant=constants.POLICY_INFRA_TENANT):
        return self.update(
            virtual_server_id, application_profile_id=application_profile_id,
            tenant=tenant)

    def update_virtual_server_persistence_profile(
            self, virtual_server_id, lb_persistence_profile_id=IGNORE,
            tenant=constants.POLICY_INFRA_TENANT):
        return self.update(
            virtual_server_id,
            lb_persistence_profile_id=lb_persistence_profile_id,
            tenant=tenant)

    def update_virtual_server_client_ssl_profile_binding(
            self, virtual_server_id, client_ssl_profile_binding=IGNORE,
            tenant=constants.POLICY_INFRA_TENANT):
        return self.update(
            virtual_server_id,
            client_ssl_profile_binding=client_ssl_profile_binding,
            tenant=tenant)

    def remove_virtual_server_client_ssl_profile_binding(
            self, virtual_server_id, tenant=constants.POLICY_INFRA_TENANT):
        lbvs_def = self._get_and_update_def(
            virtual_server_id=virtual_server_id, tenant=tenant)
        body = lbvs_def.body if lbvs_def.body else {}
        body.pop('client_ssl_profile_binding', None)
        # Server ssl profile binding can not exist without client ssl profile
        # binding
        body.pop('server_ssl_profile_binding', None)
        if body:
            lbvs_def.set_obj_dict(body)
        self.policy_api.create_or_update(
            lbvs_def, partial_updates=False)

    def remove_dlb_virtual_server_persistence_profile(
            self, virtual_server_id, tenant=constants.POLICY_INFRA_TENANT):
        dlb_vs_def = self._get_and_update_def(
            virtual_server_id=virtual_server_id, tenant=tenant)
        body = dlb_vs_def.body if dlb_vs_def.body else {}
        body.pop('lb_persistence_profile_path', None)
        if body:
            dlb_vs_def.set_obj_dict(body)
        self.policy_api.create_or_update(
            dlb_vs_def, partial_updates=False)

    def update_virtual_server_with_vip(self, virtual_server_id, vip,
                                       tenant=constants.POLICY_INFRA_TENANT):
        return self.update(
            virtual_server_id, ip_address=vip, tenant=tenant)

    def build_client_ssl_profile_binding(self, default_certificate_path,
                                         sni_certificate_paths=None,
                                         ssl_profile_path=None,
                                         client_auth_ca_paths=None,
                                         client_auth=None):
        return lb_defs.ClientSSLProfileBindingDef(
            default_certificate_path,
            sni_certificate_paths=sni_certificate_paths,
            ssl_profile_path=ssl_profile_path,
            client_auth_ca_paths=client_auth_ca_paths, client_auth=client_auth)

    def update_client_ssl_profile_binding(
            self, virtual_server_id, default_certificate_path,
            sni_certificate_paths=None, ssl_profile_path=None,
            client_auth_ca_paths=None, client_auth=None,
            tenant=constants.POLICY_INFRA_TENANT):
        client_ssl_def = lb_defs.ClientSSLProfileBindingDef(
            default_certificate_path,
            sni_certificate_paths=sni_certificate_paths,
            ssl_profile_path=ssl_profile_path,
            client_auth_ca_paths=client_auth_ca_paths, client_auth=client_auth)

        return self.update(
            virtual_server_id, client_ssl_profile_binding=client_ssl_def,
            tenant=tenant)

    def build_lb_rule(self, actions=None, display_name=None,
                      match_conditions=None, match_strategy=None, phase=None):
        return lb_defs.LBRuleDef(
            actions, match_conditions, display_name, match_strategy, phase)

    def _add_rule_in_position(self, body, lb_rule, position):
        lb_rules = body.get('rules', [])
        if position < 0 or position > len(lb_rules):
            # Add as the last one
            lb_rules.append(lb_rule)
        elif position <= len(lb_rules):
            lb_rules.insert(position, lb_rule)

        return lb_rules

    def add_lb_rule(self, virtual_server_id, actions=None,
                    name=None, match_conditions=None,
                    match_strategy=None, phase=None, position=-1,
                    tenant=constants.POLICY_INFRA_TENANT):
        lb_rule = lb_defs.LBRuleDef(
            actions, match_conditions, name, match_strategy, phase)
        lbvs_def = self.entry_def(
            virtual_server_id=virtual_server_id,
            tenant=tenant)
        body = self.policy_api.get(lbvs_def)
        lb_rules = self._add_rule_in_position(body, lb_rule, position)
        return self._update(virtual_server_id=virtual_server_id,
                            vs_data=body,
                            rules=lb_rules, tenant=tenant)

    def update_lb_rules(self, virtual_server_id, rules,
                        tenant=constants.POLICY_INFRA_TENANT):
        lbvs_def = self.entry_def(
            virtual_server_id=virtual_server_id,
            tenant=tenant)
        lbvs_path = lbvs_def.get_resource_path()

        @utils.retry_upon_exception(
            nsxlib_exc.StaleRevision,
            max_attempts=self.policy_api.client.max_attempts)
        def _update():
            # Get the current data of vs
            lbvs_body = self.policy_api.get(lbvs_def)
            lbvs_body['rules'] = copy.deepcopy(rules)
            # Update the backend using PUT
            self.policy_api.client.update(lbvs_path, lbvs_body)

        _update()

    def update_lb_rule(self, virtual_server_id, lb_rule_name,
                       actions=None, match_conditions=None,
                       match_strategy=None, phase=None, position=-1,
                       compare_name_suffix=None,
                       tenant=constants.POLICY_INFRA_TENANT):
        lb_rule = lb_defs.LBRuleDef(
            actions, match_conditions, lb_rule_name, match_strategy, phase)
        lbvs_def = self.entry_def(
            virtual_server_id=virtual_server_id,
            tenant=tenant)
        body = self.policy_api.get(lbvs_def)
        lb_rules = body.get('rules', [])

        # Remove existing rule
        try:
            if compare_name_suffix:
                rule_index = next(lb_rules.index(r) for r in lb_rules
                                  if r.get('display_name',
                                           '').endswith(compare_name_suffix))
            else:
                rule_index = next(lb_rules.index(r) for r in lb_rules
                                  if r.get('display_name') == lb_rule_name)
        except Exception:
            err_msg = (_("No resource in rules matched for values: "
                         "%(values)s") % {'values': lb_rule_name})
            raise nsxlib_exc.ResourceNotFound(
                manager=self,
                operation=err_msg)
        if position < 0:
            position = rule_index

        del(lb_rules[rule_index])

        # Insert new rule
        lb_rules = self._add_rule_in_position(body, lb_rule, position)
        return self._update(
            virtual_server_id=virtual_server_id,
            rules=lb_rules, vs_data=body, tenant=tenant)

    def remove_lb_rule(self, virtual_server_id, lb_rule_name,
                       check_name_suffix=False,
                       tenant=constants.POLICY_INFRA_TENANT):
        lbvs_def = self.entry_def(virtual_server_id=virtual_server_id,
                                  tenant=tenant)
        body = self.policy_api.get(lbvs_def)
        lb_rules = body.get('rules', [])
        if check_name_suffix:
            lb_rules = [r for r in lb_rules
                        if not r.get('display_name', '').endswith(
                            lb_rule_name)]
        else:
            lb_rules = [r for r in lb_rules
                        if r.get('display_name') != lb_rule_name]
        return self._update(
            virtual_server_id=virtual_server_id, vs_data=body,
            rules=lb_rules, tenant=tenant)

    def build_access_list_control(self, action, group_path, enabled=None):
        return lb_defs.LBAccessListControlDef(action, group_path, enabled)

    def get_path(self, virtual_server_id,
                 tenant=constants.POLICY_INFRA_TENANT):
        profile_def = self.entry_def(
            virtual_server_id=virtual_server_id,
            tenant=tenant)
        return profile_def.get_resource_full_path()

    def wait_until_realized(self, virtual_server_id, entity_type=None,
                            tenant=constants.POLICY_INFRA_TENANT,
                            sleep=None, max_attempts=None):
        lbvs_def = self.entry_def(
            virtual_server_id=virtual_server_id, tenant=tenant)
        return self._wait_until_realized(
            lbvs_def, entity_type=entity_type,
            sleep=sleep, max_attempts=max_attempts)


class NsxPolicyLBMonitorProfileBase(NsxPolicyResourceBase):
    """NSX Policy LB monitor profile"""

    def create_or_overwrite(self,
                            lb_monitor_profile_id=None,
                            tags=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT,
                            **kwargs):
        lb_monitor_profile_id = self._init_obj_uuid(lb_monitor_profile_id)
        lb_monitor_profile_def = self._init_def(
            lb_monitor_profile_id=lb_monitor_profile_id,
            tags=tags,
            tenant=tenant,
            **kwargs)
        self._create_or_store(lb_monitor_profile_def)
        return lb_monitor_profile_id

    def delete(self, lb_monitor_profile_id,
               tenant=constants.POLICY_INFRA_TENANT):
        lb_monitor_profile_def = self.entry_def(
            lb_monitor_profile_id=lb_monitor_profile_id,
            tenant=tenant)
        self._delete_with_retry(lb_monitor_profile_def)

    def get(self, lb_monitor_profile_id,
            tenant=constants.POLICY_INFRA_TENANT, silent=False):
        lb_monitor_profile_def = self.entry_def(
            lb_monitor_profile_id=lb_monitor_profile_id,
            tenant=tenant)
        return self.policy_api.get(lb_monitor_profile_def, silent=silent)

    def list(self, tenant=constants.POLICY_INFRA_TENANT):
        lb_monitor_profile_def = self.entry_def(tenant=tenant)
        return self._list(lb_monitor_profile_def)

    def update(self,
               lb_monitor_profile_id,
               name=IGNORE,
               tags=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT,
               **kwargs):
        self._update(
            lb_monitor_profile_id=lb_monitor_profile_id,
            name=name,
            tags=tags,
            tenant=tenant,
            **kwargs)

    def get_path(self, lb_monitor_profile_id,
                 tenant=constants.POLICY_INFRA_TENANT):
        mon_def = self.entry_def(lb_monitor_profile_id=lb_monitor_profile_id,
                                 tenant=tenant)
        return mon_def.get_resource_full_path()


class NsxPolicyLBMonitorProfileHttpApi(NsxPolicyLBMonitorProfileBase):
    """NSX Policy LB HTTP monitor profile"""

    def create_or_overwrite(self,
                            lb_monitor_profile_id=None,
                            tags=IGNORE,
                            name=IGNORE,
                            interval=IGNORE,
                            timeout=IGNORE,
                            fall_count=IGNORE,
                            rise_count=IGNORE,
                            monitor_port=IGNORE,
                            request_url=IGNORE,
                            request_method=IGNORE,
                            request_version=IGNORE,
                            request_headers=IGNORE,
                            request_body=IGNORE,
                            response_status_codes=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxPolicyLBMonitorProfileHttpApi,
                     self).create_or_overwrite(
            lb_monitor_profile_id=lb_monitor_profile_id,
            tags=tags,
            name=name,
            interval=interval,
            timeout=timeout,
            fall_count=fall_count,
            rise_count=rise_count,
            monitor_port=monitor_port,
            request_url=request_url,
            request_method=request_method,
            request_version=request_version,
            request_headers=request_headers,
            request_body=request_body,
            response_status_codes=response_status_codes,
            tenant=tenant)

    def update(self,
               lb_monitor_profile_id,
               tags=IGNORE,
               name=IGNORE,
               interval=IGNORE,
               timeout=IGNORE,
               fall_count=IGNORE,
               rise_count=IGNORE,
               monitor_port=IGNORE,
               request_url=IGNORE,
               request_method=IGNORE,
               request_version=IGNORE,
               request_headers=IGNORE,
               request_body=IGNORE,
               response_status_codes=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxPolicyLBMonitorProfileHttpApi, self).update(
            lb_monitor_profile_id=lb_monitor_profile_id,
            tags=tags,
            name=name,
            interval=interval,
            timeout=timeout,
            fall_count=fall_count,
            rise_count=rise_count,
            monitor_port=monitor_port,
            request_url=request_url,
            request_method=request_method,
            request_version=request_version,
            request_headers=request_headers,
            request_body=request_body,
            response_status_codes=response_status_codes,
            tenant=tenant)

    @property
    def entry_def(self):
        return lb_defs.LBHttpMonitorProfileDef


class NsxPolicyLBMonitorProfileHttpsApi(NsxPolicyLBMonitorProfileHttpApi):
    """NSX Policy LB HTTPS monitor profile"""

    @property
    def entry_def(self):
        return lb_defs.LBHttpsMonitorProfileDef


class NsxPolicyLBMonitorProfileUdpApi(NsxPolicyLBMonitorProfileBase):
    """NSX Policy LB UDP monitor profile"""

    def create_or_overwrite(self,
                            lb_monitor_profile_id=None,
                            tags=IGNORE,
                            name=IGNORE,
                            interval=IGNORE,
                            timeout=IGNORE,
                            fall_count=IGNORE,
                            rise_count=IGNORE,
                            monitor_port=IGNORE,
                            receive=IGNORE,
                            send=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxPolicyLBMonitorProfileUdpApi,
                     self).create_or_overwrite(
            lb_monitor_profile_id=lb_monitor_profile_id,
            tags=tags,
            name=name,
            interval=interval,
            timeout=timeout,
            fall_count=fall_count,
            rise_count=rise_count,
            monitor_port=monitor_port,
            receive=receive,
            send=send,
            tenant=tenant)

    def update(self,
               lb_monitor_profile_id,
               tags=IGNORE,
               name=IGNORE,
               interval=IGNORE,
               timeout=IGNORE,
               fall_count=IGNORE,
               rise_count=IGNORE,
               monitor_port=IGNORE,
               receive=IGNORE,
               send=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxPolicyLBMonitorProfileUdpApi, self).update(
            lb_monitor_profile_id=lb_monitor_profile_id,
            tags=tags,
            name=name,
            interval=interval,
            timeout=timeout,
            fall_count=fall_count,
            rise_count=rise_count,
            monitor_port=monitor_port,
            receive=receive,
            send=send,
            tenant=tenant)

    @property
    def entry_def(self):
        return lb_defs.LBUdpMonitorProfileDef


class NsxPolicyLBMonitorProfileIcmpApi(NsxPolicyLBMonitorProfileBase):
    """NSX Policy LB ICMP monitor profile"""

    def create_or_overwrite(self,
                            lb_monitor_profile_id=None,
                            tags=IGNORE,
                            name=IGNORE,
                            interval=IGNORE,
                            timeout=IGNORE,
                            fall_count=IGNORE,
                            rise_count=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxPolicyLBMonitorProfileIcmpApi,
                     self).create_or_overwrite(
            lb_monitor_profile_id=lb_monitor_profile_id,
            tags=tags,
            name=name,
            interval=interval,
            timeout=timeout,
            fall_count=fall_count,
            rise_count=rise_count,
            tenant=tenant)

    def update(self,
               lb_monitor_profile_id,
               tags=IGNORE,
               name=IGNORE,
               interval=IGNORE,
               timeout=IGNORE,
               fall_count=IGNORE,
               rise_count=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxPolicyLBMonitorProfileIcmpApi, self).update(
            lb_monitor_profile_id=lb_monitor_profile_id,
            tags=tags,
            name=name,
            interval=interval,
            timeout=timeout,
            fall_count=fall_count,
            rise_count=rise_count,
            tenant=tenant)

    @property
    def entry_def(self):
        return lb_defs.LBIcmpMonitorProfileDef


class NsxPolicyLBMonitorProfileTcpApi(NsxPolicyLBMonitorProfileBase):
    """NSX Policy LB TCP monitor profile"""

    def create_or_overwrite(self,
                            lb_monitor_profile_id=None,
                            tags=IGNORE,
                            name=IGNORE,
                            interval=IGNORE,
                            timeout=IGNORE,
                            fall_count=IGNORE,
                            rise_count=IGNORE,
                            monitor_port=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxPolicyLBMonitorProfileTcpApi,
                     self).create_or_overwrite(
            lb_monitor_profile_id=lb_monitor_profile_id,
            tags=tags,
            name=name,
            interval=interval,
            timeout=timeout,
            fall_count=fall_count,
            rise_count=rise_count,
            monitor_port=monitor_port,
            tenant=tenant)

    def update(self,
               lb_monitor_profile_id,
               tags=IGNORE,
               name=IGNORE,
               interval=IGNORE,
               timeout=IGNORE,
               fall_count=IGNORE,
               rise_count=IGNORE,
               monitor_port=IGNORE,
               tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxPolicyLBMonitorProfileTcpApi, self).update(
            lb_monitor_profile_id=lb_monitor_profile_id,
            tags=tags,
            name=name,
            interval=interval,
            timeout=timeout,
            fall_count=fall_count,
            rise_count=rise_count,
            monitor_port=monitor_port,
            tenant=tenant)

    @property
    def entry_def(self):
        return lb_defs.LBTcpMonitorProfileDef


class NsxPolicyLoadBalancerApi(object):
    """This is the class that have all load balancer policy apis"""
    def __init__(self, *args):
        self.lb_http_profile = NsxPolicyLBAppProfileHttpApi(*args)
        self.lb_fast_tcp_profile = NsxPolicyLBAppProfileFastTcpApi(*args)
        self.lb_fast_udp_profile = NsxPolicyLBAppProfileFastUdpApi(*args)
        self.client_ssl_profile = (
            NsxPolicyLoadBalancerClientSSLProfileApi(*args))
        self.server_ssl_profile = (
            NsxPolicyLoadBalancerServerSSLProfileApi(*args))
        self.lb_persistence_profile = (
            NsxPolicyLoadBalancerPersistenceProfileApi(*args))
        self.lb_cookie_persistence_profile = (
            NsxPolicyLoadBalancerCookiePersistenceProfileApi(*args))
        self.lb_source_ip_persistence_profile = (
            NsxPolicyLoadBalancerSourceIpPersistenceProfileApi(*args))
        self.lb_service = NsxPolicyLoadBalancerServiceApi(*args)
        self.virtual_server = NsxPolicyLoadBalancerVirtualServerAPI(*args)
        self.lb_pool = NsxPolicyLoadBalancerPoolApi(*args)
        self.lb_monitor_profile_http = NsxPolicyLBMonitorProfileHttpApi(*args)
        self.lb_monitor_profile_https = (
            NsxPolicyLBMonitorProfileHttpsApi(*args))
        self.lb_monitor_profile_udp = NsxPolicyLBMonitorProfileUdpApi(*args)
        self.lb_monitor_profile_icmp = NsxPolicyLBMonitorProfileIcmpApi(*args)
        self.lb_monitor_profile_tcp = NsxPolicyLBMonitorProfileTcpApi(*args)



# Copyright 2017 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from vmware_nsxlib.tests.unit.v3.policy import policy_testcase
from vmware_nsxlib.v3 import nsx_constants
from vmware_nsxlib.v3.policy import constants
from vmware_nsxlib.v3.policy import core_defs as policy


class TestPolicyDomain(policy_testcase.TestPolicyApi):

    def test_create(self):
        domain_def = policy.DomainDef(
            domain_id='archaea',
            name='prokaryotic cells',
            description='typically characterized by membrane lipids')
        self.policy_api.create_or_update(domain_def)
        self.assert_json_call('PATCH', self.client,
                              'infra/domains/archaea',
                              data=domain_def.get_obj_dict())

    def test_delete(self):
        domain_def = policy.DomainDef(domain_id='bacteria')
        self.policy_api.delete(domain_def)
        self.assert_json_call('DELETE', self.client,
                              'infra/domains/bacteria')

    def test_get(self):
        domain_def = policy.DomainDef(domain_id='eukarya')
        self.policy_api.get(domain_def)
        self.assert_json_call('GET', self.client,
                              'infra/domains/eukarya')

    def test_list(self):
        domain_def = policy.DomainDef()
        self.policy_api.list(domain_def)
        self.assert_json_call('GET', self.client, 'infra/domains')


class TestPolicyGroup(policy_testcase.TestPolicyApi):

    def test_create(self):
        group_def = policy.GroupDef(
            domain_id='eukarya',
            group_id='cats',
            name='felis catus')
        self.policy_api.create_or_update(group_def)
        self.assert_json_call('PATCH', self.client,
                              'infra/domains/eukarya/groups/cats',
                              data=group_def.get_obj_dict())

    def test_create_with_domain(self):
        domain_def = policy.DomainDef(domain_id='eukarya',
                                      name='eukarya',
                                      description='dude with cell membranes')
        group_def = policy.GroupDef(domain_id='eukarya',
                                    group_id='cats',
                                    name='Ailuropoda melanoleuca')

        self.policy_api.create_with_parent(domain_def, group_def)
        data = domain_def.get_obj_dict()
        data['groups'] = [group_def.get_obj_dict()]
        self.assert_json_call('PATCH', self.client,
                              'infra/domains/eukarya',
                              data=data)

    def test_create_with_single_tag(self):
        domain_def = policy.DomainDef(domain_id='eukarya')
        group_def = policy.GroupDef(domain_id='eukarya', group_id='dogs',
                                    conditions=policy.Condition('spaniel'))
        self.policy_api.create_with_parent(domain_def, group_def)
        data = domain_def.get_obj_dict()
        data['groups'] = [group_def.get_obj_dict()]

        # validate body structure and defaults
        expected_condition = {'value': 'spaniel',
                              'operator': 'EQUALS',
                              'member_type': 'LogicalPort',
                              'resource_type': 'Condition',
                              'key': 'Tag'}
        expected_group = {'id': 'dogs',
                          'resource_type': 'Group',
                          'expression': [expected_condition]}
        expected_data = {'id': 'eukarya',
                         'resource_type': 'Domain',
                         'groups': [expected_group]}
        self.assert_json_call('PATCH', self.client,
                              'infra/domains/eukarya',
                              data=expected_data)

    def test_create_with_multi_tag(self):
        domain_def = policy.DomainDef(domain_id='eukarya')
        pines = policy.Condition(
            'pine',
            operator=constants.CONDITION_OP_CONTAINS)
        maples = policy.Condition(
            'maple',
            operator=constants.CONDITION_OP_STARTS_WITH)
        group_def = policy.GroupDef(domain_id='eukarya', group_id='trees',
                                    conditions=[pines, maples])
        self.policy_api.create_with_parent(domain_def, group_def)
        data = domain_def.get_obj_dict()
        data['groups'] = [group_def.get_obj_dict()]
        self.assert_json_call('PATCH', self.client,
                              'infra/domains/eukarya',
                              data=data)

    def test_delete(self):
        group_def = policy.GroupDef(domain_id='eukarya', group_id='giraffe')
        self.policy_api.delete(group_def)
        self.assert_json_call('DELETE', self.client,
                              'infra/domains/eukarya/groups/giraffe')


class TestPolicyService(policy_testcase.TestPolicyApi):

    def test_create(self):
        service_def = policy.ServiceDef(service_id='roomservice')
        self.policy_api.create_or_update(service_def)
        self.assert_json_call('PATCH', self.client,
                              'infra/services/roomservice',
                              data=service_def.get_obj_dict())

    def test_create_l4_with_parent(self):
        service_def = policy.ServiceDef(service_id='roomservice')
        entry_def = policy.L4ServiceEntryDef(service_id='roomservice',
                                             protocol='TCP',
                                             entry_id='http',
                                             name='room http',
                                             dest_ports=[80, 8080])

        self.policy_api.create_with_parent(service_def, entry_def)
        expected_entry = {'id': 'http',
                          'resource_type': 'L4PortSetServiceEntry',
                          'display_name': 'room http',
                          'l4_protocol': 'TCP',
                          'destination_ports': [80, 8080]}
        expected_data = {'id': 'roomservice',
                         'resource_type': 'Service',
                         'service_entries': [expected_entry]}
        self.assert_json_call('PATCH', self.client,
                              'infra/services/roomservice',
                              data=expected_data)

    def test_create_icmp_with_parent(self):
        service_def = policy.ServiceDef(name='icmpservice',
                                        service_id='icmpservice')
        entry_def = policy.IcmpServiceEntryDef(service_id='icmpservice',
                                               version=4,
                                               entry_id='icmp',
                                               name='icmpv4')

        self.policy_api.create_with_parent(service_def, entry_def)
        expected_entry = {'id': 'icmp',
                          'resource_type': 'ICMPTypeServiceEntry',
                          'display_name': 'icmpv4',
                          'protocol': 'ICMPv4'}
        expected_data = {'id': 'icmpservice',
                         'resource_type': 'Service',
                         'display_name': 'icmpservice',
                         'service_entries': [expected_entry]}
        self.assert_json_call('PATCH', self.client,
                              'infra/services/icmpservice',
                              data=expected_data)

    def test_create_mixed_with_parent(self):
        service_def = policy.ServiceDef(name='mixedservice',
                                        service_id='mixedservice')
        l4_entry_def = policy.L4ServiceEntryDef(service_id='mixedservice',
                                                protocol='TCP',
                                                entry_id='http',
                                                name='http',
                                                dest_ports=[80, 8080])
        icmp_entry_def = policy.IcmpServiceEntryDef(service_id='mixedservice',
                                                    version=4,
                                                    entry_id='icmp',
                                                    name='icmpv4')

        self.policy_api.create_with_parent(service_def,
                                           [l4_entry_def, icmp_entry_def])

        expected_l4_entry = {'id': 'http',
                             'resource_type': 'L4PortSetServiceEntry',
                             'display_name': 'http',
                             'l4_protocol': 'TCP',
                             'destination_ports': [80, 8080]}
        expected_icmp_entry = {'id': 'icmp',
                               'resource_type': 'ICMPTypeServiceEntry',
                               'display_name': 'icmpv4',
                               'protocol': 'ICMPv4'}
        expected_data = {'id': 'mixedservice',
                         'resource_type': 'Service',
                         'display_name': 'mixedservice',
                         'service_entries': [
                             expected_l4_entry, expected_icmp_entry]}
        self.assert_json_call('PATCH', self.client,
                              'infra/services/mixedservice',
                              data=expected_data)


class TestPolicyCommunicationMap(policy_testcase.TestPolicyApi):

    def setUp(self):
        super(TestPolicyCommunicationMap, self).setUp()
        self.entry1 = policy.CommunicationMapEntryDef(
            domain_id='d1',
            map_id='cm1',
            entry_id='en1',
            action='ALLOW',
            sequence_number=12,
            source_groups=["group1",
                           "group2"],
            dest_groups=["group1"],
            service_ids=["service1"],
            direction=nsx_constants.IN_OUT)

        self.entry2 = policy.CommunicationMapEntryDef(
            domain_id='d1',
            map_id='cm2',
            entry_id='en2',
            action='ALLOW',
            sequence_number=13,
            source_groups=["group1",
                           "group2"],
            dest_groups=["group3"],
            service_ids=["service2"],
            direction=nsx_constants.IN)

        self.expected_data1 = {'id': 'en1',
                               'resource_type': 'Rule',
                               'sequence_number': 12,
                               'action': 'ALLOW',
                               'source_groups':
                               ['/infra/domains/d1/groups/group1',
                                '/infra/domains/d1/groups/group2'],
                               'destination_groups':
                               ['/infra/domains/d1/groups/group1'],
                               'services':
                               ['/infra/services/service1'],
                               'direction': 'IN_OUT'}

        self.expected_data2 = {'id': 'en2',
                               'resource_type': 'Rule',
                               'sequence_number': 13,
                               'action': 'ALLOW',
                               'source_groups':
                               ['/infra/domains/d1/groups/group1',
                                '/infra/domains/d1/groups/group2'],
                               'destination_groups':
                               ['/infra/domains/d1/groups/group3'],
                               'services':
                               ['/infra/services/service2'],
                               'direction': 'IN'}

    def test_create_with_one_entry(self):
        map_def = policy.CommunicationMapDef(domain_id='d1', map_id='cm1')

        self.policy_api.create_with_parent(map_def, self.entry1)
        expected_data = map_def.get_obj_dict()
        expected_data['rules'] = [self.expected_data1]
        self.assert_json_call('PATCH', self.client,
                              'infra/domains/d1/security-policies/cm1',
                              data=expected_data)

    def test_create_with_two_entries(self):
        map_def = policy.CommunicationMapDef(domain_id='d1', map_id='cm1')

        self.policy_api.create_with_parent(map_def,
                                           [self.entry1, self.entry2])
        expected_data = map_def.get_obj_dict()
        expected_data['rules'] = [self.expected_data1,
                                  self.expected_data2]
        self.assert_json_call('PATCH', self.client,
                              'infra/domains/d1/security-policies/cm1',
                              data=expected_data)

    def test_update_entry(self):
        self.policy_api.create_or_update(self.entry1)

        self.assert_json_call('PATCH', self.client,
                              'infra/domains/d1/security-policies/cm1/'
                              'rules/en1',
                              data=self.expected_data1)

    def test_delete_entry(self):
        self.policy_api.delete(self.entry2)

        self.assert_json_call('DELETE', self.client,
                              'infra/domains/d1/security-policies/cm2/'
                              'rules/en2')


class TestPolicyEnforcementPoint(policy_testcase.TestPolicyApi):

    def test_create(self):
        ep_def = policy.EnforcementPointDef(ep_id='ep1', name='The Point',
                                            ip_address='1.1.1.1',
                                            username='admin',
                                            password='a')

        self.policy_api.create_or_update(ep_def)
        ep_path = policy.EnforcementPointDef(ep_id='ep1').get_resource_path()
        self.assert_json_call('PATCH', self.client,
                              ep_path,
                              data=ep_def.get_obj_dict())


class TestPolicyTransportZone(policy_testcase.TestPolicyApi):

    def test_get(self):
        tz_def = policy.TransportZoneDef(tz_id='tz1', ep_id='default')
        self.policy_api.get(tz_def)
        tz_path = tz_def.get_resource_path()
        self.assert_json_call('GET', self.client, tz_path)


class TestPolicyEdgeCluster(policy_testcase.TestPolicyApi):

    def test_get(self):
        ec_def = policy.EdgeClusterDef(ec_id='ec1', ep_id='default')
        self.policy_api.get(ec_def)
        ec_path = ec_def.get_resource_path()
        self.assert_json_call('GET', self.client, ec_path)


class TestPolicyDeploymentMap(policy_testcase.TestPolicyApi):

    def test_create(self):
        map_def = policy.DeploymentMapDef(map_id='dm1',
                                          domain_id='d1',
                                          ep_id='ep1')

        self.policy_api.create_or_update(map_def)
        ep_path = policy.EnforcementPointDef(
            ep_id='ep1').get_resource_full_path()
        expected_data = {'id': 'dm1',
                         'resource_type': 'DeploymentMap',
                         'enforcement_point_path': ep_path}

        self.assert_json_call('PATCH', self.client,
                              'infra/domains/d1/domain-deployment-maps/dm1',
                              data=expected_data)


class TestPolicyTier1(policy_testcase.TestPolicyApi):

    def test_create(self):
        name = 'test'
        description = 'desc'
        tier0_id = '000'
        tier1_id = '111'
        route_adv = policy.RouteAdvertisement(static_routes=True,
                                              subnets=True,
                                              nat=True,
                                              lb_vip=False,
                                              lb_snat=False)
        ipv6_ndra_profile_id = '111'
        tier1_def = policy.Tier1Def(
            tier1_id=tier1_id,
            name=name, description=description,
            route_advertisement=route_adv,
            tier0=tier0_id,
            ipv6_ndra_profile_id=ipv6_ndra_profile_id)
        expected_data = {"id": "%s" % tier1_id,
                         "resource_type": "Tier1",
                         "description": "%s" % description,
                         "display_name": "%s" % name,
                         "tier0_path": "/infra/tier-0s/%s" % tier0_id,
                         "route_advertisement_types": route_adv.get_obj_dict(),
                         "ipv6_profile_paths": ["/infra/ipv6-ndra-profiles/"
                                                "%s" % ipv6_ndra_profile_id]}
        self.policy_api.create_or_update(tier1_def)
        tier1_path = tier1_def.get_resource_path()
        self.assert_json_call('PATCH', self.client,
                              tier1_path,
                              data=expected_data)

    def test_create_no_ipv6_profile(self):
        name = 'test'
        description = 'desc'
        tier0_id = '000'
        tier1_id = '111'
        route_adv = policy.RouteAdvertisement(static_routes=True,
                                              subnets=True,
                                              nat=True,
                                              lb_vip=False,
                                              lb_snat=False)
        ipv6_ndra_profile_id = None
        tier1_def = policy.Tier1Def(
            tier1_id=tier1_id,
            name=name, description=description,
            route_advertisement=route_adv,
            tier0=tier0_id,
            ipv6_ndra_profile_id=ipv6_ndra_profile_id)
        expected_data = {"id": "%s" % tier1_id,
                         "resource_type": "Tier1",
                         "description": "%s" % description,
                         "display_name": "%s" % name,
                         "tier0_path": "/infra/tier-0s/%s" % tier0_id,
                         "route_advertisement_types": route_adv.get_obj_dict(),
                         "ipv6_profile_paths": ["/infra/ipv6-ndra-profiles/"
                                                "default"]}
        self.policy_api.create_or_update(tier1_def)
        tier1_path = tier1_def.get_resource_path()
        self.assert_json_call('PATCH', self.client,
                              tier1_path,
                              data=expected_data)



# Copyright 2021 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from vmware_nsxlib.v3.policy import core_defs

AVI_AUTH_TOKEN_PATH = "infra/alb-auth-token"


class AlbAuthTokenProvider(object):

    def __init__(self, policy_api):
        self.policy_api = policy_api

    def get_avi_lb_auth_token(self, username, hours=1):
        body = {'username': username, 'hours': hours}
        return self.policy_api.update(AVI_AUTH_TOKEN_PATH, body)

    def get_avi_endpoint_info(self):
        enforcement_point_path = (core_defs.AVI_ENDPOINT_PATTERN % 'infra')
        return self.policy_api.get(enforcement_point_path)
